/*!
 * @file      apm32f0xx_crc.c
 *
 * @brief     This file provides all the CRC firmware functions
 *
 * @version   V1.0.0
 *
 * @date      2020-05-11
 *
 */

#include "apm32f0xx_crc.h"

/** @addtogroup Peripherals_Library Standard Peripheral Library
  @{
*/

/** @addtogroup CRC_Driver CRC Driver
  @{
*/

/** @addtogroup CRC_Fuctions Fuctions
  @{
*/

/*!
 * @brief     Config CRC peripheral registers to their default reset values
 *
 * @param     None
 *
 * @retval    None
 *
 * @note
 */
void CRC_Rest(void)
{
    CRC->DATA = 0xFFFFFFFF;
    CRC->ID   = 0x00;
    CRC->INIT = 0xFFFFFFFF;
    CRC->CTRL = 0x00000001;
}

/*!
 * @brief     Reset CRC data register (DATA)
 *
 * @param     None
 *
 * @retval    None
 *
 * @note
 */
void CRC_ResetDATA(void)
{
    CRC->CTRL_B.RST = BIT_SET;
}

/*!
 * @brief     Selects the reverse operation to be performed on input data
 *
 * @param     revInData:   Reverse input data
 *                         The parameter can be combination of following values:
 *                         @arg CRC_REVERSE_INPUT_DATA_NO:   Bit order not affected
 *                         @arg CRC_REVERSE_INPUT_DATA_8B:   Bit reversal done by byte
 *                         @arg CRC_REVERSE_INPUT_DATA_16B:  Bit reversal done by half-word
 *                         @arg CRC_REVERSE_INPUT_DATA_32B:  Bit reversal done by word
 *
 * @retval    None
 *
 * @note
 */
void CRC_SelectReverseInputData(CRC_REVERSE_INPUT_DATA_T revInData)
{
    CRC->CTRL_B.RID = revInData;
}

/*!
 * @brief     Enable the reverse operation on output data
 *
 * @param     None
 *
 * @retval    None
 *
 * @note
 */
void CRC_EnableReverseOutputData(void)
{
    CRC->CTRL_B.ROD = BIT_SET;
}

/*!
 * @brief     Disable the reverse operation on output data
 *
 * @param     None
 *
 * @retval    None
 *
 * @note
 */
void CRC_DisableReverseOutputData(void)
{
    CRC->CTRL_B.ROD = BIT_RESET;
}

/*!
 * @brief     Initializes the INIT register.
 *
 * @param     initValue: Programmable initial CRC value
 *
 * @retval    None
 *
 * @note
 */
void CRC_WriteInitRegister(uint32_t initValue)
{
    CRC->INIT = initValue;
}

/*!
 * @brief     Calculate a 32-bit CRC for a given data word (32 bits)
 *
 * @param     data: data word(32-bit) to compute its CRC
 *
 * @retval    32-bit CRC
 *
 * @note
 */
uint32_t CRC_CalculateCRC(uint32_t data)
{
    CRC->DATA = data;

    return (CRC->DATA);
}

/*!
 * @brief     Computes the 32-bit CRC of a given buffer of data word(32-bit)
 *
 * @param     pBuffer: Pointer to the buffer containing the data to be computed
 *
 * @param     bufferLength: buffer length
 *
 * @retval    32-bit CRC
 *
 * @note
 */
uint32_t CRC_CalculateBlockCRC(uint32_t pBuffer[], uint32_t bufferLength)
{
    uint32_t index = 0;

    for (index = 0; index < bufferLength; index++)
    {
        CRC->DATA = pBuffer[index];
    }

    return (CRC->DATA);
}

/*!
 * @brief     Returns the current CRC value
 *
 * @param     None
 *
 * @retval    32-bit CRC
 *
 * @note
 */
uint32_t CRC_ReadCRC(void)
{
    return (CRC->DATA);
}

/*!
 * @brief     Stores a 8-bit data in the Independent Data(ID) register
 *
 * @param     IDValue: 8-bit value to be stored in the ID register
 *
 * @retval    None
 *
 * @note
 */
void CRC_WriteIDRegister(uint8_t IDValue)
{
    CRC->ID = IDValue;
}

/*!
 * @brief      Returns a 8-bit data stored in the Independent Data(ID) register
 *
 * @param      None
 *
 * @retval     8-bit value of the ID register
 *
 * @note
 */
uint8_t CRC_ReadIDRegister(void)
{
    return (CRC->ID);
}

/**@} end of group CRC_Fuctions*/
/**@} end of group CRC_Driver */
/**@} end of group Peripherals_Library*/
