/*!
 * @file        apm32f0xx_rtc.c
 *
 * @brief       This file provides firmware functions to manage the following
 *              functionalities of the Real-Time Clock (RTC) peripheral
 *
 * @version     V1.0.0
 *
 * @date        2020-5-20
 *
 */

#include "apm32f0xx_rtc.h"
#include "apm32f0xx_rcm.h"

/** @addtogroup Peripherals_Library Standard Peripheral Library
  @{
*/

/** @addtogroup RTC_Driver RTC Driver
  @{
*/

/** @addtogroup RTC_Fuctions Fuctions
  @{
*/

static uint8_t RTC_ByteConBcd2(uint8_t val);
static uint8_t RTC_Bcd2ConByte(uint8_t val);

/*!
 * @brief       Deinitializes the RTC registers to their default reset values
 *
 * @param       None
 *
 * @retval      SUCCESS or ERROR
 *
 * @note
 */
uint8_t RTC_Reset(void)
{
    RTC_DisableWriteProtection();

    if (RTC_EnableInit() == ERROR)
    {
        RTC_EnableWriteProtection();
        return ERROR;
    }
    else
    {
        RTC->TIME     = (uint32_t)0x00000000;
        RTC->AUTORLD  = (uint32_t)0x0000FFFF;
        RTC->DATE     = (uint32_t)0x00002101;
        RTC->CTRL    &= (uint32_t)0x00000000;
        RTC->PSC      = (uint32_t)0x007F00FF;
        RTC->ALARM    = (uint32_t)0x00000000;
        RTC->SHIFT    = (uint32_t)0x00000000;
        RTC->CAL      = (uint32_t)0x00000000;
        RTC->ASS      = (uint32_t)0x00000000;

        RTC->STS = (uint32_t)0x00000007;
        RTC->TAFCFG = 0x00000000;

        if (RTC_WaitForSynchro() == ERROR)
        {
            RTC_EnableWriteProtection();
            return ERROR;
        }
        else
        {
            RTC_EnableWriteProtection();
            return SUCCESS;
        }
    }
}

/*!
 * @brief       Deinitializes the RTC registers to their default reset values
 *
 * @param       Struct : pointer to a RTC_Config_T structure which will be initialized
 *
 * @retval      SUCCESS or ERROR
 *
 * @note
 */
uint8_t RTC_Config(RTC_Config_T* Struct)
{
    RTC_DisableWriteProtection();

    if (RTC_EnableInit() == ERROR)
    {
        RTC_EnableWriteProtection();
        return ERROR;
    }
    else
    {
        RTC->CTRL_B.FM  = (Struct->format);
        RTC->PSC_B.SPF = (Struct->SynchPrediv);
        RTC->PSC_B.APF = (Struct->AsynchPrediv);

        RTC_DisableInit();
        RTC_EnableWriteProtection();
        return SUCCESS;
    }
}

/*!
 * @brief       Fills each RTC_ConfigStruct member with its default value
 *
* @param        Struct : pointer to a RTC_Config_T structure which will be initialized
 *
 * @retval      None
 *
 * @note
 */
void RTC_ConfigStructInit(RTC_Config_T* Struct)
{
    Struct->format = RTC_HourFormat_24;
    Struct->AsynchPrediv = (uint32_t)0x7F;
    Struct->SynchPrediv = (uint32_t)0xFF;
}

/*!
 * @brief       Enable the write protection for RTC registers
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void RTC_EnableWriteProtection(void)
{
    RTC->WRTPROT = 0xFF;
}

/*!
 * @brief       Disable the write protection for RTC registers
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void RTC_DisableWriteProtection(void)
{
    RTC->WRTPROT = 0xCA;
    RTC->WRTPROT = 0x53;
}

/*!
 * @brief       Enable the RTC Initialization mode.
 *
 * @param       None
 *
 * @retval      SUCCESS or ERROR
 *
 * @note
 */
uint8_t RTC_EnableInit(void)
{
    __IO uint32_t cnt = 0x00;
    uint32_t initstatus = 0x00;

    if (RTC->STS_B.INTF == BIT_RESET)
    {
        RTC->STS = (uint32_t)0xFFFFFFFF;

        do
        {
            initstatus = RTC->STS_B.INTF;
            cnt++;
        }
        while ((cnt != RTC_INITMODE_TIMEOUT) && (initstatus == 0x00));

        if (RTC->STS_B.INTF != BIT_RESET)
        {
            return SUCCESS;
        }
        else
        {
            return ERROR;
        }
    }
    else
    {
        return SUCCESS;
    }
}

/*!
 * @brief       Disable the RTC Initialization mode.
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void RTC_DisableInit(void)
{
    RTC->STS_B.INTMOD = BIT_RESET;
}

/*!
 * @brief       Waits until the RTC Time and Date registers (RTC_TIME and RTC_DATA) are
 *              synchronized with RTC APB clock
 * @param       None
 *
 * @retval      SUCCESS or ERROR
 *
 * @note
 */
uint8_t RTC_WaitForSynchro(void)
{
    __IO uint32_t cnt = 0x00;
    uint32_t synchrostatus = 0x00;

    if (RTC->CTRL_B.BTS == BIT_RESET)
    {
        return SUCCESS;
    }
    else
    {
        RTC_DisableWriteProtection();
        RTC->STS &= (uint32_t)0xFFFFFF5F;

        do
        {
            synchrostatus = RTC->STS_B.RSF;
            cnt++;
        }
        while ((cnt != RTC_SYNCHRO_TIMEOUT) && (synchrostatus == 0x00));

        if (RTC->STS_B.RSF != BIT_RESET)
        {
            RTC_EnableWriteProtection();
            return SUCCESS;
        }
        else
        {
            RTC_EnableWriteProtection();
            return ERROR;
        }
    }
}

/*!
 * @brief       Enables the RTC reference clock detection
 *
 * @param       None
 *
 * @retval      SUCCESS or ERROR
 *
 * @note
 */
uint8_t RTC_EnableRefClock(void)
{
    RTC_DisableWriteProtection();

    if (RTC_EnableInit() == ERROR)
    {
        RTC_EnableWriteProtection();
        return ERROR;
    }
    else
    {
        RTC->CTRL_B.RCDEN = BIT_SET;
        RTC_DisableInit();
        RTC_EnableWriteProtection();
        return SUCCESS;
    }
}

/*!
 * @brief       Disable the RTC reference clock detection
 *
 * @param       None
 *
 * @retval      SUCCESS or ERROR
 *
 * @note
 */
uint8_t RTC_DisableRefClock(void)
{
    RTC_DisableWriteProtection();

    if (RTC_EnableInit() == ERROR)
    {
        RTC_EnableWriteProtection();
        return ERROR;
    }
    else
    {
        RTC->CTRL_B.RCDEN = BIT_RESET;
        RTC_DisableInit();
        RTC_EnableWriteProtection();
        return SUCCESS;
    }
}

/*!
 * @brief       Enable the RTC reference clock detection
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void RTC_EnableBypassShadow(void)
{
    RTC_DisableWriteProtection();
    RTC->CTRL_B.BTS = BIT_SET;
    RTC_EnableWriteProtection();
}

/*!
 * @brief       Disable the RTC reference clock detection
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void RTC_DisableBypassShadow(void)
{
    RTC_DisableWriteProtection();
    RTC->CTRL_B.BTS = BIT_RESET;
    RTC_EnableWriteProtection();
}

/*!
 * @brief       Config the RTC current time
 *
 * @param       Format: specifies the Format to write
 *                This parameter can be one of the following values:
 *                @arg RTC_FORMAT_BIN: Format in Bin
 *                @arg RTC_FORMAT_BCD: Format in BCD
 *
 * @param       TimeStruct:  Pointer to a RTC_TIME_T structure that
 *                          contains the configuration information for the RTC peripheral
 *
 * @retval      None
 *
 * @note
 */
uint8_t RTC_ConfigTime(RTC_FORMAT_T Format, RTC_TIME_T* TimeStruct)
{
    uint8_t state = ERROR;
    uint32_t temp = 0;

    if (Format == RTC_FORMAT_BIN)
    {
        if (RTC->CTRL_B.FM == BIT_RESET)
        {
            TimeStruct->H12 = RTC_H12_AM;
        }
    }
    else
    {
        if (RTC->CTRL_B.FM == BIT_RESET)
        {
            TimeStruct->H12 = RTC_H12_AM;
        }
        else
        {
            temp = RTC_Bcd2ConByte(TimeStruct->hours);
        }
    }

    if (Format != RTC_FORMAT_BIN)
    {
        temp = (((uint32_t)(TimeStruct->hours) << 16) | \
                ((uint32_t)(TimeStruct->minutes) << 8) | \
                ((uint32_t)(TimeStruct->seconds))| \
                ((uint32_t)(TimeStruct->H12) << 22));
    }
    else
    {
        temp = (((uint32_t)RTC_ByteConBcd2(TimeStruct->hours) << 16) | \
                ((uint32_t)RTC_ByteConBcd2(TimeStruct->minutes) << 8) | \
                ((uint32_t)RTC_ByteConBcd2(TimeStruct->seconds))| \
                ((uint32_t)(TimeStruct->H12) << 22));
    }

    RTC_DisableWriteProtection();

    if (RTC_EnableInit() == ERROR)
    {
        state = ERROR;
    }
    else
    {
        RTC->TIME = (uint32_t)(temp & 0x007F7F7F);
        RTC_DisableInit();

        if (RTC->CTRL_B.BTS == RESET)
        {
            if (RTC_WaitForSynchro() == ERROR)
            {
                state = ERROR;
            }
            else
            {
                state = SUCCESS;
            }
        }
        else
        {
            state = SUCCESS;
        }
    }

    RTC_EnableWriteProtection();
    return state;
}

/*!
 * @brief       Fills each RTC_TimeStruct member with its default value
 *
 * @param       TimeStruct:  Pointer to a RTC_TIME_T structure that
 *                          contains the configuration information for the RTC peripheral
 *
 * @retval      None
 *
 * @note
 */
void RTC_TimeConfigStructInit(RTC_TIME_T* TimeStruct)
{
    /** Alarm Time Settings : Time = 00h:00mn:00sec */
    TimeStruct->hours = 0;
    TimeStruct->minutes = 0;
    TimeStruct->seconds = 0;
    TimeStruct->H12 = RTC_H12_AM;
}

/*!
 * @brief       Read the RTC current Time
 *
 * @param       Format: specifies the Format to write
 *                This parameter can be one of the following values:
 *                @arg RTC_FORMAT_BIN: Format in Bin
 *                @arg RTC_FORMAT_BCD: Format in BCD
 *
 * @param       TimeStruct:  Pointer to a RTC_TIME_T structure that
 *                          contains the configuration information for the RTC peripheral
 *
 *
 * @retval      None
 *
 * @note
 */
void RTC_ReadTime(RTC_FORMAT_T Format, RTC_TIME_T* TimeStruct)
{
    uint32_t temp = 0;
    temp = (uint32_t)((RTC->TIME) & 0x007F7F7F);

    TimeStruct->hours   = (uint8_t)((temp & 0x003F0000) >> 16);
    TimeStruct->minutes = (uint8_t)((temp & 0x00007F00) >>8);
    TimeStruct->seconds = (uint8_t)(temp &  0x0000007F);
    TimeStruct->H12 = (uint8_t)((temp & 0x00400000) >> 22);

    if (Format == RTC_FORMAT_BIN)
    {
        TimeStruct->hours   = (uint8_t)RTC_Bcd2ConByte(TimeStruct->hours);
        TimeStruct->minutes = (uint8_t)RTC_Bcd2ConByte(TimeStruct->minutes);
        TimeStruct->seconds = (uint8_t)RTC_Bcd2ConByte(TimeStruct->seconds);
    }
}

/*!
 * @brief       Read the RTC current Calendar Subseconds value
 *
 * @param       None
 *
 * @retval      RTC current Calendar Subseconds value
 *
 * @note
 */
uint32_t RTC_ReadSubSecond(void)
{
    uint32_t temp = 0;
    temp = (uint32_t)(RTC->SUBSEC);
    (void) (RTC->DATE);
    return (temp);
}

/*!
 * @brief       Config the RTC current time
 *
 * @param       Format: specifies the Format to write
 *                This parameter can be one of the following values:
 *                @arg RTC_FORMAT_BIN: Format in Bin
 *                @arg RTC_FORMAT_BCD: Format in BCD
 *
 * @param       DateStruct:  Pointer to a RTC_DATE_T structure that
 *                          contains the configuration DATE information for the RTC peripheral
 *
 *
 * @retval      None
 *
 * @note
 */
uint8_t RTC_ConfigDate(RTC_FORMAT_T Format, RTC_DATE_T* DateStruct)
{
    uint8_t state = ERROR;
    uint32_t temp = 0;

    if (Format != RTC_FORMAT_BIN)
    {
        temp = RTC_Bcd2ConByte(DateStruct->month);
        temp = RTC_Bcd2ConByte(DateStruct->date);
    }

    if (Format != RTC_FORMAT_BIN)
    {
        temp = (((uint32_t)(DateStruct->year) << 16) | \
                ((uint32_t)(DateStruct->month) << 8) | \
                ((uint32_t)(DateStruct->date))| \
                ((uint32_t)(DateStruct->weekday) << 13));
    }
    else
    {
        temp = (((uint32_t)RTC_ByteConBcd2(DateStruct->year) << 16) | \
                ((uint32_t)RTC_ByteConBcd2(DateStruct->month) << 8) | \
                ((uint32_t)RTC_ByteConBcd2(DateStruct->date))| \
                ((uint32_t)(DateStruct->weekday) << 13));

    }

    RTC_DisableWriteProtection();

    if (RTC_EnableInit() == ERROR)
    {
        state = ERROR;
    }
    else
    {
        RTC->DATE = (uint32_t)(temp & 0x00FFFF3F);
        RTC_DisableInit();

        if (RTC->CTRL_B.BTS == RESET)
        {
            if (RTC_WaitForSynchro() == ERROR)
            {
                state = ERROR;
            }
            else
            {
                state = SUCCESS;
            }
        }
        else
        {
            state = SUCCESS;
        }
    }

    RTC_EnableWriteProtection();
    return state;
}

/*!
 * @brief       Fills each RTC_DateStruct member with its default value
 *
 * @param       DateStruct:  Pointer to a RTC_DATE_T structure that
 *                          contains the configuration DATE information for the RTC peripheral
 * @retval      None
 *
 * @note
 */
void RTC_DateConfigStructInit(RTC_DATE_T* DateStruct)
{
    DateStruct->weekday = RTC_WEEKDAY_MONDAY;
    DateStruct->month = RTC_MONTH_JANUARY;
    DateStruct->date = 1;
    DateStruct->year = 0;
}

/*!
 * @brief       the RTC current date
 *
 * @param       Format: specifies the Format to write
 *                This parameter can be one of the following values:
 *                @arg RTC_FORMAT_BIN: Format in Bin
 *                @arg RTC_FORMAT_BCD: Format in BCD
 *
 * @param       DateStruct:  Pointer to a RTC_DATE_T structure that
 *                          contains the configuration DATE information for the RTC peripheral
 *
 *
 * @retval      None
 *
 * @note
 */
void RTC_ReadDate(RTC_FORMAT_T Format, RTC_DATE_T* DateStruct)
{
    uint32_t temp = 0;
    temp = (uint32_t)((RTC->DATE) & 0x00FFFF3F);

    DateStruct->year  = (uint8_t)((temp & 0x00FF0000 ) >> 16);
    DateStruct->month = (uint8_t)((temp & 0x00001F00) >>8);
    DateStruct->date  = (uint8_t)(temp &  0x0000003F);
    DateStruct->weekday =(uint8_t)((temp & 0x0000E000) >> 13);

    if (Format == RTC_FORMAT_BIN)
    {
        DateStruct->year  = (uint8_t)RTC_Bcd2ConByte(DateStruct->year);
        DateStruct->month = (uint8_t)RTC_Bcd2ConByte(DateStruct->month);
        DateStruct->date  = (uint8_t)RTC_Bcd2ConByte(DateStruct->date);
        DateStruct->weekday = (uint8_t)(DateStruct->weekday);
    }
}

/*!
 * @brief       Config the specified RTC Alarm
 *
 * @param       Format: specifies the Format to write
 *                This parameter can be one of the following values:
 *                @arg RTC_FORMAT_BIN: Format in Bin
 *                @arg RTC_FORMAT_BCD: Format in BCD
 *
 * @param       AlarmStruct:  Pointer to a RTC_ALARM_T structure that
 *                          contains the configuration ALARM information for the RTC peripheral
 *
 *
 * @retval      None
 *
 * @note
 */
void RTC_ConfigAlarm(RTC_FORMAT_T Format, RTC_ALARM_T* AlarmStruct)
{
    uint32_t temp = 0;

    if (Format != RTC_FORMAT_BCD)
    {
        if (RTC->CTRL_B.FM == BIT_RESET)
        {
            AlarmStruct->time.H12 = 0x00;
        }
    }
    else
    {
        if (RTC->CTRL_B.FM == BIT_RESET)
        {
            AlarmStruct->time.H12 = 0x00;
        }
        else
        {
            temp = RTC_Bcd2ConByte(AlarmStruct->time.hours);
        }

        if (AlarmStruct->AlarmDateWeekDaySel == RTC_WEEKDAY_SEL_DATE)
        {
            temp = RTC_ByteConBcd2(AlarmStruct->AlarmDateWeekDay);
        }
        else
        {
            temp = RTC_ByteConBcd2(AlarmStruct->AlarmDateWeekDay);
        }
    }

    if (Format == RTC_FORMAT_BCD)
    {
        temp = (((uint32_t)(AlarmStruct->time.hours) << 16) | \
                ((uint32_t)(AlarmStruct->time.minutes) << 8) | \
                ((uint32_t)AlarmStruct->time.seconds)| \
                ((uint32_t)(AlarmStruct->time.H12) << 22) | \
                ((uint32_t)(AlarmStruct->AlarmDateWeekDay) << 24) | \
                ((uint32_t)AlarmStruct->AlarmDateWeekDaySel << 30) | \
                ((uint32_t)AlarmStruct->AlarmMask));
    }
    else
    {
        temp = (((uint32_t)RTC_ByteConBcd2(AlarmStruct->time.hours) << 16) | \
                ((uint32_t)RTC_ByteConBcd2(AlarmStruct->time.minutes) << 8) | \
                ((uint32_t)RTC_ByteConBcd2(AlarmStruct->time.seconds))| \
                ((uint32_t)(AlarmStruct->time.H12) << 22) | \
                ((uint32_t)RTC_ByteConBcd2(AlarmStruct->AlarmDateWeekDay) << 24) | \
                ((uint32_t)AlarmStruct->AlarmDateWeekDaySel << 30) | \
                ((uint32_t)AlarmStruct->AlarmMask));
    }

    RTC_DisableWriteProtection();
    RTC->ALARM=temp;
    RTC_EnableWriteProtection();
}

/*!
 * @brief       Fills each AlarmStruct member with its default value
 *
 * @param       AlarmStruct:  Pointer to a RTC_ALARM_T structure that
 *                          contains the configuration ALARM information for the RTC peripheral
 *
 * @retval      None
 *
 * @note
 */
void RTC_AlarmConfigStructInit(RTC_ALARM_T* AlarmStruct)
{
    AlarmStruct->time.hours = 1;
    AlarmStruct->time.minutes = 2;
    AlarmStruct->time.seconds = 3;
    AlarmStruct->time.H12 = RTC_H12_AM;
    AlarmStruct->AlarmDateWeekDay = 1;
    AlarmStruct->AlarmDateWeekDaySel = RTC_WEEKDAY_SEL_DATE;
    AlarmStruct->AlarmMask= RTC_MASK_NONE;
}

/*!
 * @brief       Get the RTC Alarm value and masks
 *
 * @param       Format: specifies the Format to write
 *                This parameter can be one of the following values:
 *                @arg RTC_FORMAT_BIN: Format in Bin
 *                @arg RTC_FORMAT_BCD: Format in BCD
 *
 * @param       AlarmStruct:  Pointer to a RTC_ALARM_T structure that
 *                          contains the configuration ALARM information for the RTC peripheral
 *
 *
 * @retval      None
 *
 * @note
 */
void RTC_ReadAlarm(RTC_FORMAT_T Format, RTC_ALARM_T* AlarmStruct)
{
    AlarmStruct->time.hours   = (uint8_t)(((RTC->ALARM_B.HRT << 0x04) |RTC->ALARM_B.HRU));
    AlarmStruct->time.minutes = (uint8_t)(((RTC->ALARM_B.MINT << 0x04) | RTC->ALARM_B.MINU));
    AlarmStruct->time.seconds = (uint8_t)(((RTC->ALARM_B.SECT << 0x04) | RTC->ALARM_B.SECU));
    AlarmStruct->time.H12     = (uint8_t)(RTC->ALARM_B.APM);
    AlarmStruct->AlarmDateWeekDay = (uint8_t)(((RTC->ALARM_B.DAYT << 0x04) |RTC->ALARM_B.DAYU));
    AlarmStruct->AlarmDateWeekDaySel = (RTC_WEEKDAY_SEL_T)(RTC->ALARM_B.WDS);
    AlarmStruct->AlarmMask=(uint8_t)(((((((RTC->ALARM_B.MSK4)<<8)| (RTC->ALARM_B.MSK3))<<8)| \
                                       (RTC->ALARM_B.MSK2))<<8)|  (RTC->ALARM_B.MSK2));

    if (Format == RTC_FORMAT_BIN)
    {
        AlarmStruct->time.hours = (uint8_t)RTC_Bcd2ConByte(AlarmStruct->time.hours);
        AlarmStruct->time.minutes = (uint8_t)RTC_Bcd2ConByte(AlarmStruct->time.minutes);
        AlarmStruct->time.seconds = (uint8_t)RTC_Bcd2ConByte(AlarmStruct->time.seconds);
        AlarmStruct->AlarmDateWeekDay = (uint8_t)RTC_Bcd2ConByte(AlarmStruct->AlarmDateWeekDay);
    }
}

/*!
 * @brief       Disable the RTC reference clock detection
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void RTC_EnableAlarm(void)
{
    RTC_DisableWriteProtection();
    RTC->CTRL_B.ALREN = BIT_SET;
    RTC_EnableWriteProtection();
}

/*!
 * @brief       Disable the RTC reference clock detection
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
uint8_t RTC_DisableAlarm(void)
{
    __IO uint32_t count = 0x00;
    RTC_DisableWriteProtection();
    RTC->CTRL_B.ALREN = BIT_RESET;

    while ((count != RTC_INITMODE_TIMEOUT) && ((RTC->STS_B.AWF) == BIT_RESET))
    {
        count++;
    }

    if ((RTC->STS_B.AWF) == BIT_RESET)
    {
        RTC_EnableWriteProtection();
        return ERROR;
    }
    else
    {
        RTC_EnableWriteProtection();
        return SUCCESS;
    }
}

/*!
 * @brief       Read the RTC Alarm Subseconds value
 *
 * @param       val: specifies the value for Alarm Sub Second
 *                   this value must less than 0x00007FFF
 *
 * @param       mask: specifies the mask for Alarm Sub Second
 *                   this value must less than 0x0f
 *
 * @retval      None
 *
 * @note
 */
void RTC_ConfigAlarmSubSecond(uint32_t val, uint8_t mask)
{
    RTC_DisableWriteProtection();
    RTC->ASS_B.SSV = val;
    RTC->ASS_B.SSM = mask;
    RTC_EnableWriteProtection();
}

/*!
 * @brief       Read the RTC Alarm Subseconds value
 *
 * @param       None
 *
 * @retval      RTC Alarm Subseconds value
 *
 * @note
 */
uint32_t RTC_ReadAlarmSubSecond(void)
{
    return (uint32_t)(RTC->ASS_B.SSV);
}

/*!
 * @brief       Adds or substract one hour from the current time
 *
 * @param       sav: specifies the DayLightSaving 
 *                   This parameter can be any combination of the following values:
 *                     @arg RTC_DLS_SUB1H 
 *                     @arg RTC_DLS_ADD1H
 *
 * @param       bit: specifies the DayLightSaving
 *                    This parameter can be any combination of the following values:
 *                     @arg RTC_SO_RESET 
 *                     @arg RTC_SO_SET
 *
 * @retval      None
 *
 * @note
 */
void RTC_ConfigDayLightSaving(RTC_DAYLIGHT_SAVING_T sav, RTC_STORE_OPERATION_T bit)
{
    RTC_DisableWriteProtection();

    if (sav==RTC_DLS_ADD1H)
    {
        RTC->CTRL_B.ADD1H = (uint32_t)BIT_SET;
        RTC->CTRL_B.LES1H = (uint32_t)BIT_RESET;
    }
    else
    {
        RTC->CTRL_B.ADD1H = (uint32_t)BIT_RESET;
        RTC->CTRL_B.LES1H = (uint32_t)BIT_SET;
    }

    RTC->CTRL_B.BAK = (uint32_t)bit;
    RTC_EnableWriteProtection();
}

/*!
 * @brief       Returns the RTC Day Light Saving stored operation
 *
 * @param       None
 *
 * @retval      RTC Day Light Saving stored operation
 *
 * @note
 */
uint32_t RTC_ReadStoreOperation(void)
{
    return (RTC->CTRL_B.BAK);
}

/*!
 * @brief       Enables the RTC TimeStamp functionality with the
 *              specified time stamp pin stimulating edge
 *
 * @param       opsel: specifies RTC TimeStamp functionality
 *                   This parameter can be any combination of the following values:
 *                     @arg RTC_OPSEL_DISABLE 
 *                     @arg RTC_OPSEL_ALARMA
 *
 * @param       opp:specified time stamp pin stimulating edge
 *                    This parameter can be any combination of the following values:
 *                     @arg RTC_OPP_HIGH 
 *                     @arg RTC_OPP_LOW
 *
 *
 * @retval      None
 *
 * @note
 */
void RTC_ConfigOutput(RTC_OPSEL_T opsel, RTC_OPP_T opp)
{
    RTC_DisableWriteProtection();
    RTC->CTRL_B.OUTSEL = (uint32_t)opsel;
    RTC->CTRL_B.OUTPOL = (uint32_t)opp;
    RTC_EnableWriteProtection();
}

/*!
 * @brief       Enable the RTC clock to be output through the relative pin
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void RTC_EnableCalibOutput(void)
{
    RTC_DisableWriteProtection();
    RTC->CTRL_B.COEN = BIT_SET;
    RTC_EnableWriteProtection();
}

/*!
 * @brief       Disable the RTC clock to be output through the relative pin
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void RTC_DisableCalibOutput(void)
{
    RTC_DisableWriteProtection();
    RTC->CTRL_B.COEN = BIT_RESET;
    RTC_EnableWriteProtection();
}

/*!
 * @brief       Configure the Calibration Pinout Selection (1Hz or 512Hz).
 *
 * @param       calib:Select the Calibration output Selection .
 *                    This parameter can be any combination of the following values:
 *                     @arg RTC_CALIBOUTPUT_512Hz 
 *                     @arg RTC_CALIBOUTPUT_1Hz
 *
 * @retval      None
 *
 * @note
 */
void RTC_ConfigCalibOutput(RTC_CALIB_OUTPUT_T calib)
{
    RTC_DisableWriteProtection();
    RTC->CTRL_B.COS = (uint32_t)calib;
    RTC_EnableWriteProtection();
}

/*!
 * @brief     Configures the Synchronization Shift Control Settings.
 *
 * @param  Period: Select the Smooth Calibration Period.
 *          This parameter can be can be one of the following values:
 *            @arg RTC_SCP_16SEC: The smooth calibration periode is 16.
 *            @arg RTC_SCP_8SEC:  The smooth calibartion periode is 8s.
 * @param  bit: Select to Set or reset the CALP bit.
 *          This parameter can be one of the following values:
 *            @arg RTC_SCPP_RESET: Add one RTCCLK puls every 2**11 pulses.
 *            @arg RTC_SCPP_SET:   No RTCCLK pulses are added.
 * @param  Value: Select the value of CALM[8:0] bits.
 *          This parameter can be one any value from 0 to 0x000001FF.
 *
 * @retval    SUCCESS or ERROR
 *
 * @note
 */
uint8_t RTC_ConfigSmoothCalib(RTC_SCP_T Period, RTC_SCPP_T bit, uint32_t Value)
{
    uint8_t state = ERROR;
    uint32_t count = 0;

    RTC_DisableWriteProtection();

    if (RTC->STS_B.RPF != BIT_RESET)
    {
        while ((RTC->STS_B.RPF != BIT_RESET) && (count != RTC_RECALPF_TIMEOUT))
        {
            count++;
        }
    }

    if (RTC->STS_B.RPF == BIT_RESET)
    {
        RTC->CAL_B.CALMIN = (uint32_t)Value;
        RTC->CAL_B.CALADD = (uint32_t)bit;

        if (Period==RTC_SCP_16SEC)
        {
            RTC->CAL_B.CCP16 = BIT_SET;
        }
        else
        {
            RTC->CAL_B.CCP8 = BIT_SET;
        }

        state = SUCCESS;
    }
    else
    {
        state = ERROR;
    }

    RTC_EnableWriteProtection();
    return (state);
}

/*!
 * @brief       Enables the RTC TimeStamp functionality with the
 *              specified time stamp pin stimulating edge
 * @param       edge: Specifies the pin edge on which the TimeStamp is  activated.
 *                    This paramete can be one of the following:
 *                    @arg RTC_TIME_STAMPEDGE_RISING: the Time stamp event occurs on the rising 
 *                                           edge of the related pin.
 *                    @arg RTC_TIME_STAMPEDGE_FALLING: the Time stamp event occurs on the 
 *                                            falling edge of the related pin.
 * @retval      None
 *
 * @note
 */
void RTC_EnableTimeStamp(RTC_TIMESTAMP_EDGE_T edge)
{
    RTC_DisableWriteProtection();
    RTC->CTRL_B.TSEN = BIT_SET;
    RTC->CTRL_B.TSEAE = (uint32_t)edge;
    RTC_EnableWriteProtection();
}

/*!
 * @brief       Disables the RTC TimeStamp functionality with the
 *              specified time stamp pin stimulating edge
 *
 * @param       edge: Specifies the pin edge on which the TimeStamp is  activated.
 *                    This paramete can be one of the following:
 *                    @arg RTC_TIME_STAMPEDGE_RISING: the Time stamp event occurs on the rising 
 *                                           edge of the related pin.
 *                    @arg RTC_TIME_STAMPEDGE_FALLING: the Time stamp event occurs on the 
 *                                            falling edge of the related pin.
 *
 * @retval      None
 *
 * @note
 */
void RTC_DisableTimeStamp(RTC_TIMESTAMP_EDGE_T edge)
{
    RTC_DisableWriteProtection();
    RTC->CTRL_B.TSEN = BIT_RESET;
    RTC->CTRL_B.TSEAE = (uint32_t)edge;
    RTC_EnableWriteProtection();
}

/*!
 * @brief       Read the RTC TimeStamp value and masks
 *
 * @param      RTC_Format: specifies the format of the output parameters.
 *                        This parameter can be one of the following values:
 *                        @arg RTC_Format_BIN: data in Binary format 
 *                        @arg RTC_Format_BCD: data in BCD    format
 * @param      TimeStruct: pointer to a RTC_TIME_T structure that will 
 *                             contains the TimeStamp time values. 
 * @param      DateStruct: pointer to a RTC_DATE_T structure that will 
 *                             contains the TimeStamp date values.     
 *
 * @retval      None
 *
 * @note
 */
void RTC_ReadTimeDate(RTC_FORMAT_T format, RTC_TIME_T* TimeStruct, RTC_DATE_T* DateStruct)
{
    uint32_t temptime = 0, tempdate = 0;
    temptime = (uint32_t)((RTC->TST) & 0x007F7F7F);
    tempdate = (uint32_t)((RTC->TSD) & 0x00FFFF3F);

    TimeStruct->hours   = (uint8_t)((temptime & 0x003F0000 ) >> 16);
    TimeStruct->minutes = (uint8_t)((temptime & 0x00007F00) >>8);
    TimeStruct->seconds = (uint8_t)(temptime &  0x0000007F);
    TimeStruct->H12 = (uint8_t)((temptime & 0x00400000) >> 22);

    DateStruct->year  =  0;
    DateStruct->month = (uint8_t)((tempdate & 0x00001F00) >>8);
    DateStruct->date  = (uint8_t)(tempdate &  0x0000003F);
    DateStruct->weekday =(uint8_t)((tempdate & 0x0000E000) >> 13);

    if (format == RTC_FORMAT_BIN)
    {
        TimeStruct->hours   = (uint8_t)RTC_Bcd2ConByte(TimeStruct->hours);
        TimeStruct->minutes = (uint8_t)RTC_Bcd2ConByte(TimeStruct->minutes);
        TimeStruct->seconds = (uint8_t)RTC_Bcd2ConByte(TimeStruct->seconds);

        DateStruct->month = (uint8_t)RTC_Bcd2ConByte(DateStruct->month);
        DateStruct->date  = (uint8_t)RTC_Bcd2ConByte(DateStruct->date);
        DateStruct->weekday = (uint8_t)(DateStruct->weekday);
    }
}

/*!
 * @brief       Get the RTC timestamp Subseconds value
 *
 * @param       None
 *
 * @retval      RTC current timestamp Subseconds value
 *
 * @note
 */
uint32_t RTC_ReadTimeStampSubSecond(void)
{
    return (uint32_t)(RTC->TSSS);
}

/*!
 * @brief       Configures the select Tamper pin edge
 *
 * @param      tamper: Selected tamper pin.
 *                        This parameter can be any combination of the following values:
 *                        @arg RTC_TAMPER_1: Select Tamper 1.
 *                        @arg RTC_TAMPER_2: Select Tamper 2.
 * @param      trigger: Specifies the trigger on the tamper pin that stimulates tamper event. 
 *                        This parameter can be one of the following values:
 *                        @arg RTC_TAMPER_TRIGGER_RISINGEDGE: Rising Edge of the tamper pin causes tamper event.
 *                        @arg RTC_TAMPER_TRIGGER_FALLINGEDGE: Falling Edge of the tamper pin causes tamper event.
 *                        @arg RTC_TAMPER_TRIGGER_LOWLEVEL: Low Level of the tamper pin causes tamper event.
 *                        @arg RTC_TAMPER_TRIGGER_HIGHLEVEL: High Level of the tamper pin causes tamper event.
 *
 * @retval      None
 *
 * @note
 */
void RTC_ConfigTamperTrigger(RTC_TAMPER_T tamper, RTC_TAMPER_TRIGGER_T trigger)
{
    if (trigger==RTC_TAMPER_TRIGGER_RISINGEDGE)
    {
        if (tamper==RTC_TAMPER_1)
        {
            RTC->TAFCFG_B.TP1ACT=BIT_RESET;
        }
        else
        {
            RTC->TAFCFG_B.TP2ACT=BIT_RESET;
        }
    }
    else
    {
        if (tamper==RTC_TAMPER_1)
        {
            RTC->TAFCFG_B.TP1ACT=BIT_SET;
        }
        else
        {
            RTC->TAFCFG_B.TP2ACT=BIT_SET;
        }
    }
}

/*!
 * @brief       Enables the Tamper detection
 *
 * @param      tamper: Selected tamper pin.
 *                        This parameter can be any combination of the following values:
 *                        @arg RTC_TAMPER_1: Select Tamper 1.
 *                        @arg RTC_TAMPER_2: Select Tamper 2.
 *
 * @retval      None
 *
 * @note
 */
void RTC_EnableTamper(RTC_TAMPER_T tamper)
{
    if (tamper==RTC_TAMPER_1)
    {
        RTC->TAFCFG_B.TP1EN=BIT_SET;
    }
    else
    {
        RTC->TAFCFG_B.TP2EN=BIT_SET;
    }
}

/*!
 * @brief       Disables the Tamper detection
 *
 * @param      tamper: Selected tamper pin.
 *                        This parameter can be any combination of the following values:
 *                        @arg RTC_TAMPER_1: Select Tamper 1.
 *                        @arg RTC_TAMPER_2: Select Tamper 2.
 *
 * @retval      None
 *
 * @note
 */
void RTC_DisableTamper(RTC_TAMPER_T tamper)
{
    if (tamper==RTC_TAMPER_1)
    {
        RTC->TAFCFG_B.TP1EN=BIT_RESET;
    }
    else
    {
        RTC->TAFCFG_B.TP2EN=BIT_RESET;
    }
}

/*!
 * @brief       Configures the Tampers Filter
 *
  * @param     RTC_TamperFilter: Specifies the tampers filter.
  *                             This parameter can be one of the following values:
  *                             @arg RTC_TAMPER_FILTER_DISABLE: Tamper filter is disabled.
  *                             @arg RTC_TAMPER_FILTER_2SAMPLE: Tamper is activated after 2 consecutive samples at the active level 
  *                             @arg RTC_TAMPER_FILTER_4SAMPLE: Tamper is activated after 4 consecutive samples at the active level
  *                             @arg RTC_TAMPER_FILTER_8SAMPLE: Tamper is activated after 8 consecutive samples at the active level 
  *                                          
 *
 * @retval      None
 *
 * @note
 */
void RTC_ConfigFilter(RTC_TAMPER_FILTER_T filter)
{
    RTC->TAFCFG_B.FILCNT = (uint32_t)filter;
}

/*!
 * @brief       Configures the Tampers Sampling Frequency
 *
 * @param       freq: Specifies the tampers Sampling Frequency.
 *          T          his parameter can be one of the following values:
 *                    @arg RTC_TAMPERSAMPLINGFREQ_RTCCLK_DIV32768: Each of the tamper inputs are sampled
 *                                                         with a frequency =  RTCCLK / 32768
 *                    @arg RTC_TAMPERSAMPLINGFREQ_RTCCLK_DIV16384: Each of the tamper inputs are sampled
 *                                                         with a frequency =  RTCCLK / 16384
 *                    @arg RTC_TAMPERSAMPLINGFREQ_RTCCLK_DIV8192: Each of the tamper inputs are sampled
 *                                                      with a frequency =  RTCCLK / 8192
 *                    @arg RTC_TAMPERSAMPLINGFREQ_RTCCLK_DIV4096: Each of the tamper inputs are sampled
 *                                                        with a frequency =  RTCCLK / 4096
 *                    @arg RTC_TAMPERSAMPLINGFREQ_RTCCLK_DIV2048: Each of the tamper inputs are sampled
 *                                                        with a frequency =  RTCCLK / 2048
 *                    @arg RTC_TAMPERSAMPLINGFREQ_RTCCLK_DIV1024: Each of the tamper inputs are sampled
 *                                                        with a frequency =  RTCCLK / 1024
 *                    @arg RTC_TAMPERSAMPLINGFREQ_RTCCLK_DIV512: Each of the tamper inputs are sampled
 *                                                       with a frequency =  RTCCLK / 512  
 *                    @arg RTC_TAMPERSAMPLINGFREQ_RTCCLK_DIV256: Each of the tamper inputs are sampled
 *                                                       with a frequency =  RTCCLK / 256  
 *
 * @retval      None
 *
 * @note
 */
void RTC_ConfigSamplingFreq(RTC_TAMPER_SAMPLING_FREQ_T freq)
{
    RTC->TAFCFG_B.TSF = (uint32_t)freq;
}

/*!
 * @brief       Configures the Precharge Duration
 *
 * @param      duration: Specifies the Tampers Pins input  Precharge Duration.
 *                      This parameter can be one of the following values:
 *                      @arg RTC_PRECHARGEDURATION_1RTCCLK: Tamper pins are pre-charged before sampling during 1 RTCCLK cycle
 *                      @arg RTC_PRECHARGEDURATION_2RTCCLK: Tamper pins are pre-charged before sampling during 2 RTCCLK cycle
 *                      @arg RTC_PRECHARGEDURATION_4RTCCLK: Tamper pins are pre-charged before sampling during 4 RTCCLK cycle    
 *                      @arg RTC_PRECHARGEDURATION_8RTCCLK: Tamper pins are pre-charged before sampling during 8 RTCCLK cycle
 *      
 * @retval      None
 *
 * @note
 */
void RTC_PinsPrechargeDuration(RTC_PRECHARGE_DURATION_T duration)
{
    RTC->TAFCFG_B.PERDUR = (uint32_t)duration;
}

/*!
 * @brief       Enables the TimeStamp on Tamper Detection Event
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void RTC_EnableTDE(void)
{
    RTC->TAFCFG_B.TDE = BIT_SET;
}

/*!
 * @brief       Disable the TimeStamp on Tamper Detection Event
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void RTC_DisableTDE(void)
{
    RTC->TAFCFG_B.TDE = BIT_RESET;
}

/*!
 * @brief       Enable precharge of the selected Tamper pin
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void RTC_EnablePullUp(void)
{
    RTC->TAFCFG_B.PUDIS = BIT_SET;
}

/*!
 * @brief       Disable precharge of the selected Tamper pin
 *
 * @param       None
 *
 * @retval      None
 *
 * @note
 */
void RTC_DisablePullUp(void)
{
    RTC->TAFCFG_B.PUDIS = BIT_RESET;
}

/*!
 * @brief     Writes a data in RTC Backup data.
 *
 * @param   backup: RTC Backup data Register number.
 *          This parameter can be:
 *            @arg RTC_BAKP_DATA0 
 *            @arg RTC_BAKP_DATA1
 *            @arg RTC_BAKP_DATA2    
 *            @arg RTC_BAKP_DATA3 
 *            @arg RTC_BAKP_DATA4 
 * @param  data: Data to be written in the specified RTC Backup data register.                     

 * @retval    None
 *
 * @note
 */
void RTC_WriteBackup(RTC_BAKP_DATA_T backup, uint32_t data)
{
    switch (backup)
    {
        case 0:
            RTC->BAKRDATA1 = data;
            break;

        case 1:
            RTC->BAKRDATA2 = data;
            break;

        case 2:
            RTC->BAKRDATA3 = data;
            break;

        case 3:
            RTC->BAKRDATA4 = data;
            break;

        case 4:
            RTC->BAKRDATA5 = data;
            break;

        default:
            break;
    }
}

/*!
 * @brief     Reads a data in RTC Backup data.
 *
 * @param   backup: RTC Backup data Register number.
 *                  This parameter can be:
 *                   @arg RTC_BAKP_DATA0 
 *                   @arg RTC_BAKP_DATA1
 *                   @arg RTC_BAKP_DATA2    
 *                   @arg RTC_BAKP_DATA3 
 *                   @arg RTC_BAKP_DATA4 
 *
 * @retval    None
 *
 * @note
 */
uint32_t RTC_ReadBackup(RTC_BAKP_DATA_T backup)
{
    uint32_t data;

    switch (backup)
    {
        case 0:
            data = RTC->BAKRDATA1;
            break;

        case 1:
            data = RTC->BAKRDATA2;
            break;

        case 2:
            data = RTC->BAKRDATA3;
            break;

        case 3:
            data = RTC->BAKRDATA4;
            break;

        case 4:
            data = RTC->BAKRDATA5;
            break;

        default:
            break;
    }

    return (data);
}

/*!
 * @brief       Configures the RTC Output Pin mode
 *
 * @param       output: specifies the RTC Output (PC13) pin mode.
 *                      This parameter can be one of the following values:
 *                      @arg RTC_OPENDRAIN: RTC Output (PC13) is configured in 
 *                                    Open Drain mode.
 *                      @arg RTC_PUSHPULL:  RTC Output (PC13) is configured in 
 *                                    Push Pull mode.    
 *
 * @retval      None
 *
 * @note
 */
void RTC_ConfigOutputType(RTC_OUTPUT_T output)
{
    RTC->TAFCFG_B.PC13VAL = (uint32_t)output;
}

/*!
 * @brief     Configures the Synchronization Shift Control Settings.
 *
  * @param   Add1S: Select to add or not 1 second to the time Calendar.
  *                This parameter can be one of the following values :
  *                @arg RTC_SHIFTADD1S_SET: Add one second to the clock calendar. 
  *                @arg RTC_SHIFTADD1S_RESET:   No effect.
  * @param   SubFS: Select the number of Second Fractions to Substitute.
  *                This parameter can be one any value from 0 to 0x7FFF.
 *
 * @retval   SUCCESS or ERROR
 *
 * @note
 */
uint8_t RTC_ConfigSynchroShift(RTC_SHIFT_ADD1S_T Add1S, uint32_t SubFS)
{
    uint8_t state = ERROR;
    uint32_t shpfcount = 0;
    RTC_DisableWriteProtection();

    if (RTC->STS_B.SOPF != BIT_RESET)
    {
        while ((RTC->STS_B.SOPF != BIT_RESET) && (shpfcount != RTC_SHPF_TIMEOUT))
        {
            shpfcount++;
        }
    }

    if (RTC->STS_B.SOPF == BIT_RESET)
    {
        if (RTC->CTRL_B.RCDEN == BIT_RESET)
        {
            RTC->SHIFT_B.ADD1S =Add1S;
            RTC->SHIFT_B.SFS = SubFS;

            if (RTC_WaitForSynchro() == ERROR)
            {
                state = ERROR;
            }
            else
            {
                state = SUCCESS;
            }
        }
        else
        {
            state = ERROR;
        }
    }
    else
    {
        state = ERROR;
    }

    RTC_EnableWriteProtection();
    return (state);
}

/*!
 * @brief     Enable RTC interrupts.
 *
 * @param     interrupt: specifies the RTC interrupt sources to be enabled 
 *                       This parameter can be any combination of the following values:
 *                        @arg RTC_INT_ALR:  Alarm A interrupt mask
 *                        @arg RTC_INT_TS:   Time Stamp interrupt mask
 *                        @arg RTC_INT_TAMP: Tamper event interrupt mask
 *
 * @retval    None
 *
 * @note
 */
void RTC_EnableInterrupt(uint32_t interrupt)
{
    RTC_DisableWriteProtection();
    RTC->CTRL |= (uint32_t)(interrupt & ~0x00000004);
    RTC->TAFCFG |= (uint32_t)(interrupt & 0x00000004);
    RTC_EnableWriteProtection();
}

/*!
 * @brief     Disable RTC interrupts.
 *
 * @param     interrupt: specifies the RTC interrupt sources to be disable 
 *                       This parameter can be any combination of the following values:
 *                        @arg RTC_INT_ALR:  Alarm A interrupt mask
 *                        @arg RTC_INT_TS:   Time Stamp interrupt mask
 *                        @arg RTC_INT_TAMP: Tamper event interrupt mask
 *
 * @retval    None
 *
 * @note
 */
void RTC_DisableInterrupt(uint32_t interrupt)
{
    RTC_DisableWriteProtection();
    RTC->CTRL &= (uint32_t)~(interrupt & ~0x00000004);
    RTC->TAFCFG &= (uint32_t)~(interrupt & 0x00000004);
    RTC_EnableWriteProtection();
}


/*!
 * @brief     Read interrupt flag bit is set
 *
 * @param     flag: specifies the flag to read.
 *                  This parameter can be one of the following values:
 *                  @arg RTC_INT_FLAG_ALR: Alarm interrupt 
 *                  @arg RTC_INT_FLAG_TS: Time Stamp interrupt
 *                  @arg RTC_INT_FLAG_TAMP1: Tamper1 event interrupt 
 *                  @arg RTC_INT_FLAG_TAMP2: Tamper2 event interrupt 
 * @retval    The new state of flag (SET or RESET).
 *
 * @note
 */
uint8_t RTC_ReadIntFlag(RTC_INT_FLAG_T flag)
{
    uint32_t intEnable;
    uint32_t intStatus;

    if (flag & 0x04)
    {
        intEnable = (uint32_t)RTC->TAFCFG_B.TPIE;
        intStatus = (uint32_t)(RTC->STS & (flag>>4));
    }
    else
    {
        intEnable = (uint32_t)RTC->CTRL;
        intStatus = (uint32_t)(RTC->STS & (flag>>4));
    }

    if (intEnable && intStatus)
    {
        return SET;
    }

    return RESET;
}

/*!
 * @brief     Clear RTC interrupt flag bit
 *
 * @param     flag: specifies the flag to clear.
 *                  This parameter can be one of the following values:
 *                  @arg RTC_INT_FLAG_ALR: Alarm interrupt 
 *                  @arg RTC_INT_FLAG_TS: Time Stamp interrupt
 *                  @arg RTC_INT_FLAG_TAMP1: Tamper1 event interrupt 
 *                  @arg RTC_INT_FLAG_TAMP2: Tamper2 event interrupt 
 * @retval    The new state of flag (SET or RESET).
 *
 * @note
 */
void RTC_ClearIntFlag(RTC_INT_FLAG_T flag)
{
    RTC->STS &= (uint32_t) ~(flag>>4);
}


/*!
 * @brief  Checks whether the specified RTC flag is set or not.
 *
 * @param  flag: specifies the flag to check.
 *          This parameter can be one of the following values:
 *            @arg RTC_FLAG_AWF  
 *            @arg RTC_FLAG_WTWF 
 *            @arg RTC_FLAG_SOPF 
 *            @arg RTC_FLAG_ISF 
 *            @arg RTC_FLAG_RSF 
 *            @arg RTC_FLAG_INTF 
 *            @arg RTC_FLAG_ALRF 
 *            @arg RTC_FLAG_INITF 
 *            @arg RTC_FLAG_WTF 
 *            @arg RTC_FLAG_TSF 
 *            @arg RTC_FLAG_TSOF 
 *            @arg RTC_FLAG_TP1F 
 *            @arg RTC_FLAG_TP2F 
 *            @arg RTC_FLAG_RPF 
 * @retval The new state of RTC_FLAG (SET or RESET).
 *
 * @note
 */
uint8_t RTC_ReadStatusFlag(RTC_FLAG_T flag)
{
    return  (RTC->STS & flag) ? SET : RESET;
}

/*!
 * @brief  Clears the RTC's status flags.
 * @param  flag: specifies the RTC flag to clear.
 *               This parameter can be any combination of the following values:
 *              @arg RTC_FLAG_TP2F: Tamper 2 event flag
 *              @arg RTC_FLAG_TP1F: Tamper 1 event flag 
 *              @arg RTC_FLAG_TSOF: Time Stamp Overflow flag 
 *              @arg RTC_FLAG_TSF : Time Stamp event flag
 *              @arg RTC_FLAG_ALRF: Alarm A flag
 *              @arg RTC_FLAG_RSF:  Registers Synchronized flag
 *
 * @note
 */
void RTC_ClearStatusFlag(RTC_FLAG_T flag)
{
    RTC->STS &= (uint32_t)~flag;
}

/*!
 * @brief     Converts a 2 digit decimal to BCD format
 *
 * @param     val: Byte to be converted
 *
 * @retval    Converted byte
 *
 * @note
 */
static uint8_t RTC_ByteConBcd2(uint8_t val)
{
    uint8_t bcdhigh = 0;

    while (val >= 10)
    {
        bcdhigh++;
        val -= 10;
    }

    return  ((uint8_t)(bcdhigh << 4) | val);
}

/*!
 * @brief     Convert from 2 digit BCD to Binary
 *
 * @param     val: BCD value to be converted
 *
 * @retval    Converted word
 *
 * @note
 */
static uint8_t RTC_Bcd2ConByte(uint8_t val)
{
    uint8_t tmp = 0;
    tmp = ((uint8_t)(val & (uint8_t)0xF0) >> (uint8_t)0x4) * 10;
    return (tmp + (val & (uint8_t)0x0F));
}

/**@} end of group RTC_Fuctions*/
/**@} end of group RTC_Driver*/
/**@} end of group Peripherals_Library*/

