/*!
 * @file        apm32f0xx_syscfg.c
 *
 * @brief       This file contains all the functions for the SYSCFG peripheral
 *
 * @version     V1.0.0
 *
 * @date        2020-05-18
 *
 */

#include "apm32f0xx_syscfg.h"

/** @addtogroup Peripherals_Library Standard Peripheral Library
  @{
*/

/** @addtogroup SYSCFG_Driver SYSCFG Driver
  @{
*/

/** @addtogroup SYSCFG_Fuctions Fuctions
  @{
*/

/*!
 * @brief     Set SYSCFG CFG0/1 EINTCFG1/2/3/4 register to reset value
 *
 * @param     None

 * @retval    None
 *
 * @note
 */
void SYSCFG_Reset(void)
{
    SYSCFG->CFG1 &= (uint32_t) SYSCFG_CFG1_MEMMODE;
    SYSCFG->EINTCFG1 = 0;
    SYSCFG->EINTCFG2 = 0;
    SYSCFG->EINTCFG3 = 0;
    SYSCFG->EINTCFG4 = 0;
    SYSCFG->CFG2 |= (uint32_t) SYSCFG_CFG2_SRAMPEF;
}

/*!
 * @brief       SYSCFG Memory Remap selects
 *
 * @param       memory: selects the memory remapping
 *                      The parameter can be combination of following values:
 *                      @arg SYSCFG_MEMORY_FMC:    SYSCFG MemoryRemap Flash
 *                      @arg SYSCFG_MEMORY_SYSTEN:   SYSCFG MemoryRemap SystemMemory
 *                      @arg SYSCFG_MEMORY_SRAM:     SYSCFG MemoryRemap SRAM
 *
 * @retval      None
 *
 * @note
 */
void SYSCFG_MemoryRemapSelect(uint8_t memory)
{
    SYSCFG->CFG1_B.MEMM = (uint8_t)memory;
}

/*!
 * @brief       Enables SYSCFG DMA Channel Remap
 *
 * @param       channel: selects the DMA channels remap.
 *                       The parameter can be combination of following values:
 *                       @arg SYSCFG_DAMRMP_ADC:         ADC DMA remap
 *                       @arg SYSCFG_DAMRMP_USART1TX:    USART1 TX DMA remap
 *                       @arg SYSCFG_DAMRMP_USART1RX:    USART1 RX DMA remap
 *                       @arg SYSCFG_DAMRMP_TMR16:       Timer 16 DMA remap
 *                       @arg SYSCFG_DAMRMP_TMR17:       Timer 17 DMA remap
 *
 * @retval      None
 *
 * @note
 */
void SYSCFG_EnableDMAChannelRemap(uint32_t channel)
{
    SYSCFG->CFG1 |= (uint32_t)channel;
}

/*!
 * @brief       Disables SYSCFG DMA Channel Remap
 *
 * @param       channel: selects the DMA channels remap.
 *                       The parameter can be combination of following values:
 *                       @arg SYSCFG_DAMRMP_ADC:         ADC DMA remap
 *                       @arg SYSCFG_DAMRMP_USART1TX:    USART1 TX DMA remap
 *                       @arg SYSCFG_DAMRMP_USART1RX:    USART1 RX DMA remap
 *                       @arg SYSCFG_DAMRMP_TMR16:       Timer 16 DMA remap
 *                       @arg SYSCFG_DAMRMP_TMR17:       Timer 17 DMA remap
 *
 * @retval      None
 *
 * @note
 */
void SYSCFG_DisableDMAChannelRemap(uint32_t channel)
{
    SYSCFG->CFG1 &= (uint32_t)~channel;
}

/*!
 * @brief       Enables SYSCFG I2C Fast Mode Plus
 *
 * @param       pin:     selects the pin.
 *                       The parameter can be combination of following values:
 *                       @arg SYSCFG_I2C_FMP_PB6:    I2C PB6 Fast mode plus
 *                       @arg SYSCFG_I2C_FMP_PB7:    I2C PB7 Fast mode plus
 *                       @arg SYSCFG_I2C_FMP_PB8:    I2C PB8 Fast mode plus
 *                       @arg SYSCFG_I2C_FMP_PB9:    I2C PB9 Fast mode plus
 *                       @arg SYSCFG_I2C_FMP_PA9:    I2C PA9 Fast mode plus
 *                       @arg SYSCFG_I2C_FMP_PA10:   I2C PA10 Fast mode plus
 *                       @arg SYSCFG_I2C_FMP_I2C1:   PB10, PB11, PF6 and PF7
 *
 * @retval      None
 *
 * @note
 */
void SYSCFG_EnableI2CFastModePlus(uint32_t pin)
{
    SYSCFG->CFG1 |= (uint32_t)pin;
}

/*!
 * @brief       Disables SYSCFG I2C Fast Mode Plus
 *
 * @param       pin:     selects the pin.
 *                       The parameter can be combination of following values:
 *                       @arg SYSCFG_I2C_FMP_PB6:    I2C PB6 Fast mode plus
 *                       @arg SYSCFG_I2C_FMP_PB7:    I2C PB7 Fast mode plus
 *                       @arg SYSCFG_I2C_FMP_PB8:    I2C PB8 Fast mode plus
 *                       @arg SYSCFG_I2C_FMP_PB9:    I2C PB9 Fast mode plus
 *                       @arg SYSCFG_I2C_FMP_PA9:    I2C PA9 Fast mode plus
 *                       @arg SYSCFG_I2C_FMP_PA10:   I2C PA10 Fast mode plus
 *                       @arg SYSCFG_I2C_FMP_I2C1:   PB10, PB11, PF6 and PF7
 *
 * @retval      None
 *
 * @note
 */
void SYSCFG_DisableI2CFastModePlus(uint32_t pin)
{
    SYSCFG->CFG1 &= (uint32_t)~pin;
}

/*!
 * @brief       Select the modulation envelope source 
 *
 * @param       IRDAEnv: selects the envelope source
 *                       The parameter can be combination of following values:
 *                       @arg SYSCFG_IRDA_ENV_TMR16:     Timer16 as IRDA Modulation envelope source
 *                       @arg SYSCFG_IRDA_ENV_USART1:    USART1 as IRDA Modulation envelope source
 *                       @arg SYSCFG_IRDA_ENV_USART2:    USART4 as IRDA Modulation envelope source
 *
 * @retval      None
 *
 * @note
 */
void SYSCFG_SelectIRDAEnv(SYSCFG_IRDA_ENV_T IRDAEnv)
{
    SYSCFG->CFG1 &= ~(0x000000C0);
    SYSCFG->CFG1 |= (IRDAEnv);
}

/*!
 * @brief       Selects the GPIO pin used as EXTI Line.
 *
 * @param       port:   selects the port can be SYSCFG_PORT_(A/B/C/D/E/F)
 *
 * @param       pin:    selects the pin can be SYSCFG_PIN_(0..15)
 *
 * @retval      None
 *
 * @note
 */

void SYSCFG_EINTLine(SYSCFG_PORT_T port, SYSCFG_PIN_T pin)
{
    uint32_t status;

    status = (((uint32_t)0x0F) & port) << (0x04 * (pin & (uint8_t)0x03));

    if ( pin <= 0x03)
    {
        SYSCFG->EINTCFG1 = status;
    }
    else if (( 0x04 <= pin) & (pin <= 0x07))
    {
        SYSCFG->EINTCFG2 = status;
    }
    else if (( 0x08 <= pin) & (pin <= 0x0B))
    {
        SYSCFG->EINTCFG3 = status;
    }
    else if (( 0x0C <= pin) & (pin <= 0x0F))
    {
        SYSCFG->EINTCFG4 = status;
    }
}

/*!
 * @brief       Selected parameter to the break input of TMR1.
 *
 * @param       lock:   selects the configuration to break
 *                      The parameter can be combination of following values:
 *                      @arg SYSCFG_LOCK_HDEN:     Cortex-M0 Hardfault enable bit
 *                      @arg SYSCFG_LOCK_SRAMPLB:  SRAM parity lock bit
 *
 * @retval      None
 *
 * @note
 */
void SYSCFG_BreakLock(uint32_t lock)
{
    SYSCFG->CFG2_B.HDEN = 0;
    SYSCFG->CFG2_B.SRAMPLB = 0;

    if (lock == SYSCFG_LOCK_HDEN)
    {
        SYSCFG->CFG2_B.HDEN = BIT_SET;
    }
    else
    {
        SYSCFG->CFG2_B.SRAMPLB = BIT_SET;
    }
}

/*!
 * @brief       Read the specified SYSCFG flag
 *
 * @param       flag:   SRAM Parity error flag
 *                      @arg SYSCFG_CFG2_SRAMPEF
 *
 * @retval      None
 *
 * @note
 */
uint8_t SYSCFG_ReadStatusFlag(uint32_t flag)
{
    uint32_t status;

    status = (uint32_t)(SYSCFG->CFG2_B.SRAMPEF & flag);

    if (status == flag)
    {
        return SET;
    }

    return RESET;
}

/*!
 * @brief       Clear the specified SYSCFG flag
 *
 * @param       flag:   SRAM Parity error flag
 *                      @arg SYSCFG_CFG2_SRAMPEF
 *
 * @retval      None
 *
 * @note
 */
void SYSCFG_ClearStatusFlag(uint8_t flag)
{
    SYSCFG->CFG2 |= (uint32_t) flag;
}

/**@} end of group SYSCFG_Fuctions*/
/**@} end of group SYSCFG_Driver*/
/**@} end of group Peripherals_Library*/
