/*!
 * @file        apm32f0xx_tmr.c
 *
 * @brief       This file contains all the functions for the TMR peripheral
 *
 * @version     V1.0.0
 *
 * @date        2020-05-09
 *
 */

#include "apm32f0xx_tmr.h"
#include "apm32f0xx_rcm.h"

/** @addtogroup Peripherals_Library Standard Peripheral Library
  @{
*/

/** @addtogroup TMR_Driver  TMR Driver
  @{
*/

/** @addtogroup  TMR_Fuctions Fuctions
  @{
*/

/*!
 * @brief   Reset the TMR peripheral registers to their default reset values
 *
 * @param   TMRx: x can be can be 1, 3, 6, 14, 15, 16 and 17 to select Timer
 *
 * @retval  None
 *
 * @note      None
 */
void TMR_Reset(TMR_T* TMRx)
{
    if (TMRx == TMR1)
    {
        RCM_EnableAPB2PeriphClock(RCM_APB2_PERIPH_TMR1);
        RCM_DisableAPB2PeriphClock(RCM_APB2_PERIPH_TMR1);
    }
    else if (TMRx == TMR3)
    {
        RCM_EnableAPB1PeriphClock(RCM_APB1_PERIPH_TMR3);
        RCM_DisableAPB1PeriphClock(RCM_APB1_PERIPH_TMR3);

    }
    else if (TMRx == TMR6)
    {
        RCM_EnableAPB1PeriphClock(RCM_APB1_PERIPH_TMR6);
        RCM_DisableAPB1PeriphClock(RCM_APB1_PERIPH_TMR6);
    }
    else if (TMRx == TMR14)
    {
        RCM_EnableAPB1PeriphClock(RCM_APB1_PERIPH_TMR14);
        RCM_DisableAPB1PeriphClock(RCM_APB1_PERIPH_TMR14);
    }
    else if (TMRx == TMR15)
    {
        RCM_EnableAPB2PeriphClock(RCM_APB2_PERIPH_TMR15);
        RCM_DisableAPB2PeriphClock(RCM_APB2_PERIPH_TMR15);
    }
    else if (TMRx == TMR16)
    {
        RCM_EnableAPB2PeriphClock(RCM_APB2_PERIPH_TMR16);
        RCM_DisableAPB2PeriphClock(RCM_APB2_PERIPH_TMR16);
    }
    else if (TMRx == TMR17)
    {
        RCM_EnableAPB2PeriphClock(RCM_APB2_PERIPH_TMR17);
        RCM_DisableAPB2PeriphClock(RCM_APB2_PERIPH_TMR17);
    }
}

/*!
 * @brief   Initializes the TMRx Time Base Unit peripheral according to
 *          the specified parameters in the TMR_ConfigTimeBaseStructInit
 *
 * @param   TMRx: x can be can be 1, 3, 6, 14, 15, 16 and 17 to select Timer
 *
 * @param   tmrstruct: pointer to a TMR_TimeBase_T structure that contains
 *          the configuration information for the specified TMR peripheral
 *
 * @retval  None
 *
 * @note    None
 */
void TMR_ConfigTimeBase(TMR_T* TMRx, TMR_TimeBase_T* tmrConfig)
{
    if ((TMRx == TMR1) || (TMRx == TMR3))
    {
        /** Select the Counter Mode */
        TMRx->CTRL1_B.CNTDIR = tmrConfig->counterMode;
        TMRx->CTRL1_B.CNTMODE = (tmrConfig->counterMode) >> 1;
    }

    if (TMRx != TMR6)
    {
        /** Set the clock division */
        TMRx->CTRL1_B.CKDR = tmrConfig->clockDivision;
    }

    /** Set the Autoreload value */
    TMRx->AUTORLD = tmrConfig->period ;

    /** Set the Prescaler value */
    TMRx->DIV = tmrConfig->div ;

    if ((TMRx == TMR1) || (TMRx == TMR15)|| (TMRx == TMR16) || (TMRx == TMR17))
    {
        /** Set the Repetition Counter value */
        TMRx->REPCNT = tmrConfig->repetitionCounter;
    }

    /** Enable Update generation */
    TMRx->SCEG_B.UEG = BIT_SET;
}

/*!
 * @brief   Fills each TMR_ConfigTimeBaseStructInit member with its default value
 *
 * @param   tmrstruct: pointer to a TMR_TimeBase_T structure that contains
 *          the configuration information for the specified TMR peripheral
 *
 * @retval  None
 *
 * @note    None
 */
void TMR_ConfigTimeBaseStructInit(TMR_TimeBase_T* tmrstruct)
{
    /** Set the default configuration */
    tmrstruct->period = 0xFFFFFFFF;
    tmrstruct->div = 0x0000;
    tmrstruct->clockDivision = TMR_CKD_DIV1;
    tmrstruct->counterMode = TMR_COUNTER_MODE_UP;
    tmrstruct->repetitionCounter = 0x0000;
}

/*!
 * @brief   Configures the TMRx Div
 *
 * @param   TMRx: x can be can be 1, 3, 6, 14, 15, 16 and 17 to select Timer
 *
 * @param   div: specifies the Div Register value
 *
 * @param   mode: specifies the TMR Prescaler Reload mode
 *
 * @retval  None
 *
 * @note    None
 */
void TMR_ConfigDIV(TMR_T* TMRx, uint16_t div, TMR_PRESCALER_RELOAD_T mode)
{
    /** Set the DIV value */
    TMRx->DIV = div;
    /** Set or reset the UGG Bit */
    TMRx->SCEG_B.UEG  = mode;;
}

/*!
 * @brief   Specifies the TMRx Counter Mode to be used
 *
 * @param   TMRx: x can be can be 1 or 3 to select Timer
 *
 * @param   mode : specifies the Counter Mode to be used
 *
 * @retval  None
 *
 * @note    None
 */
void TMR_ConfigCounterMode(TMR_T* TMRx, TMR_COUNTER_MODE_T mode)
{
    /** Set the Counter Mode */
    TMRx->CTRL1_B.CNTDIR = mode;
    TMRx->CTRL1_B.CNTMODE = (mode) >> 1;
}

/*!
 * @brief   Sets the TMRx Counter Register value
 *
 * @param   TMRx: x can be can be 1, 3, 6, 14, 15, 16 and 17 to select Timer
 *
 * @param   counter: specifies the Counter register new value
 *
 * @retval  None
 *
 * @note      None
 */
void TMR_SetCounter(TMR_T* TMRx, uint32_t counter)
{
    /** Set the Counter Register value */
    TMRx->CNT = counter;
}

/*!
 * @brief   Read the TMRx Counter value
 *
 * @param   TMRx: x can be can be 1, 3, 6, 14, 15, 16 and 17 to select Timer
 *
 * @retval  Counter Register value
 *
 * @note    None
 */
uint32_t TMR_ReadCounter(TMR_T* TMRx)
{
    /** Read the Counter Register value */
    return (uint32_t)TMRx->CNT;
}

/*!
 * @brief     Sets the AutoReload Register value
 *
 * @param     TMRx: x can be can be 1, 3, 6, 14, 15, 16 and 17 to select Timer
 *
 * @param     autoReload: autoReload register new value
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_SetAutoreload(TMR_T* TMRx, uint16_t autoReload)
{
    TMRx->AUTORLD = autoReload;
}

/*!
 * @brief   Read the TMRx Div value
 *
 * @param   TMRx: x can be can be 1, 3, 6, 14, 15, 16 and 17 to select Timer
 *
 * @retval  Div Register value
 *
 * @note    None
 */
uint32_t TMR_ReadDiv(TMR_T* TMRx)
{
    /** Read the Div Register value */
    return (uint32_t)TMRx->DIV;
}

/*!
 * @brief     Enable the No update event
 *
 * @param     TMRx: x can be can be 1, 3, 6, 14, 15, 16 and 17 to select Timer
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_EnableNGUpdate(TMR_T* TMRx)
{
    TMRx->CTRL1_B.NGUE = ENABLE;
}

/*!
 * @brief     Enable the No update event
 *
 * @param     TMRx: x can be can be 1, 3, 6, 14, 15, 16 and 17 to select Timer
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_DisableNGUpdate(TMR_T* TMRx)
{
    TMRx->CTRL1_B.NGUE = DISABLE;
}

/*!
 * @brief     Configures the Update Request Interrupt source.
 *
 * @param     TMRx: x can be can be 1, 3, 6, 14, 15, 16 and 17 to select Timer
 *
 * @param     updateSource: Config the Update source
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_UPdateRequestConfig(TMR_T* TMRx, TMR_UPDATE_SOURCE_T updateSource)
{
    if (updateSource != TMR_UPDATE_SOURCE_GLOBAL)
    {
        TMRx->CTRL1_B.UES = BIT_SET;
    }
    else
    {
        TMRx->CTRL1_B.UES = BIT_RESET;
    }
}

/*!
 * @brief     Enables peripheral Preload register on AUTORLD
 *
 * @param     TMRx: x can be can be 1, 3, 6, 14, 15, 16 and 17 to select Timer
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_EnableAUTOReload(TMR_T* TMRx)
{
    TMRx->CTRL1_B.ARBEN = ENABLE;
}

/*!
 * @brief     Disable peripheral Preload register on AUTORLD
 *
 * @param     TMRx: x can be can be 1, 3, 6, 14, 15, 16 and 17 to select Timer
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_DisableAUTOReload(TMR_T* TMRx)
{
    TMRx->CTRL1_B.ARBEN = DISABLE;
}

/*!
 * @brief     Selects the One Pulse Mode
 *
 * @param     TMRx: x can be can be 1, 3, 6, 14, 15, 16 and 17 to select Timer
 *
 * @param     OPMode:Config OP Mode to be used
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_SelectOnePulseMode(TMR_T* TMRx, TMR_OPMODE_T OPMode)
{
    TMRx->CTRL1_B.SPMEN = OPMode;
}

/*!
 * @brief     Sets the Clock Division value
 *
 * @param     TMRx: x can be can be 1, 3, 14, 15, 16 and 17 to select Timer
 *
 * @param     clockDivision: clock division value
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_SetClockDivision(TMR_T* TMRx, TMR_CKD_T clockDivision)
{
    TMRx->CTRL1_B.CKDR = clockDivision;
}

/*!
 * @brief     Enable the specified TMR peripheral
 *
 * @param     TMRx: x can be can be 1, 3, 6, 14, 15, 16 and 17 to select Timer
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_Enable(TMR_T* TMRx)
{
    TMRx->CTRL1_B.CNTEN = ENABLE;
}

/*!
 * @brief     Disable the specified TMR peripheral
 *
 * @param     TMRx: x can be can be 1, 3, 6, 14, 15, 16 and 17 to select Timer
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_Disable(TMR_T* TMRx)
{
    TMRx->CTRL1_B.CNTEN = DISABLE;
}

/*!
 * @brief     Configures the: Break feature, dead time, Lock level, the OSSI
 *
 * @param     TMRx: x can be can be 1, 15, 16 and 17 to select Timer
 *
 * @param     structure: pointer to a TMR_BDTInit_T structure that contains
 *            the BRKCTRL Register configuration  information for the TMR peripheral
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_ConfigBDT(TMR_T* TMRx, TMR_BDTInit_T* structure)
{
    TMRx->BRKCTRL_B.IMOS = structure->IMOS_State;
    TMRx->BRKCTRL_B.RMOS = structure->RMOS_State;
    TMRx->BRKCTRL_B.PROTCFG = structure->protcfc_Level;
    TMRx->BRKCTRL_B.DTS = structure->deadTime;
    TMRx->BRKCTRL_B.BRKEN = structure->BRKEN_State;
    TMRx->BRKCTRL_B.BRKPOL = structure->BRKPolarity;
    TMRx->BRKCTRL_B.AOEN = structure->automaticOutput;
}

/*!
 * @brief     Initialize the BDT timer with its default value.
 *
 * @param     structure: pointer to a TMR_BDTInit_T structure that contains
 *            the BRKCTRL Register configuration  information for the TMR peripheral
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_ConfigBDTStructInit( TMR_BDTInit_T* structure)
{
    /** Set the default configuration */
    structure->RMOS_State = TMR_RMOS_STATE_DISABLE;
    structure->IMOS_State = TMR_IMOS_STATE_DISABLE;
    structure->protcfc_Level = TMR_PROTCFC_OFF;
    structure->deadTime = 0x00;
    structure->BRKEN_State = TMR_BRKEN_DISABLE;
    structure->BRKPolarity = TMR_BRKPOLARITY_LOW;
    structure->automaticOutput = TMR_AUTOMATIC_OUTPUT_DIAABLE;
}

/*!
 * @brief     Enable TMRx PWM output.
 *
 * @param     TMRx: x can be can be 1, 15, 16 and 17 to select Timer
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_EnablePWMOutputs(TMR_T* TMRx)
{
    TMRx->BRKCTRL_B.WOEN = ENABLE;
}

/*!
 * @brief     Disable TMRx PWM output.
 *
 * @param     TMRx: x can be can be 1, 15, 16 and 17 to select Timer
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_DisablePWMOutputs(TMR_T* TMRx)
{
    TMRx->BRKCTRL_B.WOEN = DISABLE;
}

/*!
 * @brief     Configure channel 1 according to parameters
 *
 * @param     TMRx: x can be can be 1, 3, 14, 15, 16 and 17 to select Timer
 *
 * @param     OCcongigStruct: Channel configuration structure
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_OC1Config(TMR_T* TMRx, TMR_OCConfig_T* OCcongigStruct)
{

    /** Disable the Channel 1: Reset the CH1CCEN Bit */
    TMRx->CHCTRL_B.CH1CCEN = BIT_RESET;

    /** Reset and Select the Output Compare Mode Bits */
    TMRx->CH1CCM_OUTPUT_B.MOD1SEL = BIT_RESET;
    TMRx->CH1CCM_OUTPUT_B.OC1MS = OCcongigStruct->OC_Mode;

    /** Reset and Set the Output Polarity level */
    TMRx->CHCTRL_B.CH1CCP = OCcongigStruct->OC_Polarity;

    /** Set the Output State */
    TMRx->CHCTRL_B.CH1CCEN = OCcongigStruct->OC_OutputState;

    if ((TMRx == TMR1) || (TMRx == TMR15) ||(TMRx == TMR16)
        || (TMRx == TMR17))
    {
        /** Reset and Set the Output N Polarity level */
        TMRx->CHCTRL_B.CH1OCNP = OCcongigStruct->OC_NPolarity;

        /** Reset and Set the Output N State */
        TMRx->CHCTRL_B.CH1OCNEN = OCcongigStruct->OC_OutputNState;

        /** Reset the Output Compare and Output Compare N IDLE State */
        TMRx->CTRL2_B.CH1ISO = BIT_RESET;
        TMRx->CTRL2_B.CH1NISO = BIT_RESET;

        /** Set the Output Idle state */
        TMRx->CTRL2_B.CH1ISO = OCcongigStruct->OC_Idlestate;
        /** Set the Output N State */
        TMRx->CTRL2_B.CH1NISO = OCcongigStruct->OC_NIdlestate;
    }

    /** Set the Capture Compare Register value */
    TMRx->CH1CC = OCcongigStruct->Pulse;
}

/*!
 * @brief     Configure channel 2 according to parameters
 *
 * @param     TMRx: x can be can be 1, 3  and 15 to select Timer
 *
 * @param     OCcongigStruct: Channel configuration structure
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_OC2Config(TMR_T* TMRx, TMR_OCConfig_T* OCcongigStruct)
{

    /** Disable the Channel 2: Reset the CH1CCEN Bit */
    TMRx->CHCTRL_B.CH2CCEN = BIT_RESET;

    /** Reset and Select the Output Compare Mode Bits */
    TMRx->CH1CCM_OUTPUT_B.MOD2SEL = BIT_RESET;
    TMRx->CH1CCM_OUTPUT_B.OC2MS = OCcongigStruct->OC_Mode;

    /** Reset and Set the Output Polarity level */
    TMRx->CHCTRL_B.CH2CCP = BIT_RESET;
    TMRx->CHCTRL_B.CH2CCP = OCcongigStruct->OC_Polarity;

    /** Set the Output State */
    TMRx->CHCTRL_B.CH2CCEN = OCcongigStruct->OC_OutputState;

    if (TMRx == TMR1)
    {
        /** Reset and Set the Output N Polarity level */
        TMRx->CHCTRL_B.CH2OCNP = BIT_RESET;
        TMRx->CHCTRL_B.CH2OCNP = OCcongigStruct->OC_NPolarity;

        /** Reset and Set the Output N State */
        TMRx->CHCTRL_B.CH2OCNEN = BIT_RESET;
        TMRx->CHCTRL_B.CH2OCNEN = OCcongigStruct->OC_OutputNState;

        /** Reset the Output Compare and Output Compare N IDLE State */
        TMRx->CTRL2_B.CH2ISO = BIT_RESET;
        TMRx->CTRL2_B.CH2NISO = BIT_RESET;

        /** Set the Output Idle state */
        TMRx->CTRL2_B.CH2ISO = OCcongigStruct->OC_Idlestate;
        /** Set the Output N State */
        TMRx->CTRL2_B.CH2NISO = OCcongigStruct->OC_NIdlestate;
    }

    /** Set the Capture Compare Register value */
    TMRx->CH2CC = OCcongigStruct->Pulse;
}

/*!
 * @brief     Configure channel 3 according to parameters
 *
 * @param     TMRx: x can be can be 1 or 3 to select Timer
 *
 * @param     OCcongigStruct: Channel configuration structure
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_OC3Config(TMR_T* TMRx, TMR_OCConfig_T* OCcongigStruct)
{

    /** Disable the Channel 3: Reset the CH1CCEN Bit */
    TMRx->CHCTRL_B.CH3CCEN = BIT_RESET;

    /** Reset and Select the Output Compare Mode Bits */
    TMRx->CH2CCM_OUTPUT_B.MOD3SEL = BIT_RESET;
    TMRx->CH2CCM_OUTPUT_B.OC3MS = OCcongigStruct->OC_Mode;

    /** Reset and Set the Output Polarity level */
    TMRx->CHCTRL_B.CH3CCP = BIT_RESET;
    TMRx->CHCTRL_B.CH3CCP = OCcongigStruct->OC_Polarity;

    /** Set the Output State */
    TMRx->CHCTRL_B.CH3CCEN = OCcongigStruct->OC_OutputState;

    if (TMRx == TMR1)
    {
        /** Reset and Set the Output N Polarity level */
        TMRx->CHCTRL_B.CH3OCNP = BIT_RESET;
        TMRx->CHCTRL_B.CH3OCNP = OCcongigStruct->OC_NPolarity;

        /** Reset and Set the Output N State */
        TMRx->CHCTRL_B.CH3OCNEN = BIT_RESET;
        TMRx->CHCTRL_B.CH3OCNEN = OCcongigStruct->OC_OutputNState;

        /** Reset the Output Compare and Output Compare N IDLE State */
        TMRx->CTRL2_B.CH3ISO = BIT_RESET;
        TMRx->CTRL2_B.CH3NISO = BIT_RESET;

        /** Set the Output Idle state */
        TMRx->CTRL2_B.CH3ISO = OCcongigStruct->OC_Idlestate;
        /** Set the Output N State */
        TMRx->CTRL2_B.CH3NISO = OCcongigStruct->OC_NIdlestate;
    }

    /** Set the Capture Compare Register value */
    TMRx->CH3CC = OCcongigStruct->Pulse;
}

/*!
 * @brief     Configure channel 4 according to parameters
 *
 * @param     TMRx: x can be can be 1 or 3 to select Timer
 *
 * @param     OCcongigStruct: Channel configuration structure
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_OC4Config(TMR_T* TMRx, TMR_OCConfig_T* OCcongigStruct)
{

    /** Disable the Channel 4: Reset the CH1CCEN Bit */
    TMRx->CHCTRL_B.CH4CCEN = BIT_RESET;

    /** Reset and Select the Output Compare Mode Bits */
    TMRx->CH2CCM_OUTPUT_B.MOD4SEL = BIT_RESET;
    TMRx->CH2CCM_OUTPUT_B.OC4MS = OCcongigStruct->OC_Mode;

    /** Reset and Set the Output Polarity level */
    TMRx->CHCTRL_B.CH4CCP = BIT_RESET;
    TMRx->CHCTRL_B.CH4CCP = OCcongigStruct->OC_Polarity;

    /** Set the Output State */
    TMRx->CHCTRL_B.CH4CCEN = OCcongigStruct->OC_OutputState;

    if (TMRx == TMR1)
    {
        /** Reset the Output Compare and Output Compare IDLE State */
        TMRx->CTRL2_B.CH4ISO = BIT_RESET;

        /** Set the Output Idle state */
        TMRx->CTRL2_B.CH4ISO = OCcongigStruct->OC_Idlestate;
    }

    /** Set the Capture Compare Register value */
    TMRx->CH4CC = OCcongigStruct->Pulse;
}

/*!
 * @brief     Initialize the OC timer with its default value.
 *
 * @param     OCcongigStruct: Channel configuration structure
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_OCConfigStructInit(TMR_OCConfig_T* OCcongigStruct)
{
    /** Set the default configuration */
    OCcongigStruct->OC_Mode = TMR_OC_MODE_TMRING;
    OCcongigStruct->OC_OutputState = TMR_OUTPUT_STATE_DISABLE;
    OCcongigStruct->OC_OutputNState = TMR_OUTPUT_NSTATE_DISABLE;
    OCcongigStruct->Pulse = 0x0000;
    OCcongigStruct->OC_Polarity = TMR_OC_POLARITY_HIGH;
    OCcongigStruct->OC_NPolarity = TMR_OC_NPOLARITY_HIGH;
    OCcongigStruct->OC_Idlestate = TMR_OCIDLESTATE_RESET;
    OCcongigStruct->OC_NIdlestate = TMR_OCNIDLESTATE_RESET;
}

/*!
 * @brief     Selects the Output Compare Mode.
 *
 * @param     TMRx: x can be can be 1, 3, 14, 15, 16 and 17 to select Timer
 *
 * @param     channel: specifies the TMR Channel
 *                    This parameter can be any combination of the following values:
 *                     @arg TMR_CHANNEL_1 
 *                     @arg TMR_CHANNEL_2
 *                     @arg TMR_CHANNEL_3 
 *                     @arg TMR_CHANNEL_4
 * @param     OCMode: specifies the TMR Output Compare Mode
 *                    This parameter can be any combination of the following values:
 *                     @arg TMR_OC_MODE_TMRING 
 *                     @arg TMR_OC_MODE_ACTIVE
 *                     @arg TMR_OC_MODE_INACTIVE 
 *                     @arg TMR_OC_MODE_LOWLEVEL
  *                    @arg TMR_OC_MODE_HIGHLEVEL 
 *                     @arg TMR_OC_MODE_PWM1
 *                     @arg TMR_OC_MODE_PWM2 

 * @retval    None
 *
 * @note      None
 */
void TMR_SelectOCxMode(TMR_T* TMRx, TMR_CHANNEL_T channel, TMR_OC_MODE_T mode)
{
    TMRx->CHCTRL &= BIT_RESET << channel;

    if (channel == TMR_CHANNEL_1)
    {
        TMRx->CH1CCM_OUTPUT_B.OC1MS = mode;
    }
    else if (channel == TMR_CHANNEL_2)
    {
        TMRx->CH1CCM_OUTPUT_B.OC2MS = mode;
    }
    else if (channel == TMR_CHANNEL_3)
    {
        TMRx->CH2CCM_OUTPUT_B.OC3MS = mode;
    }
    else if (channel == TMR_CHANNEL_4)
    {
        TMRx->CH2CCM_OUTPUT_B.OC4MS = mode;
    }
}

/*!
 * @brief     Sets the Capture Compare1 Register value
 *
 * @param     TMRx: x can be can be 1, 3, 14, 15, 16 and 17 to select Timer
 *
 * @param     compare: specifies the Capture Compare1 register new value
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_SetCompare1(TMR_T* TMRx, uint16_t compare)
{
    TMRx->CH1CC = compare;
}

/*!
 * @brief     Sets the Capture Compare2 Register value
 *
 * @param     TMRx: x can be can be 1, 3 or 15 to select Timer
 *
 * @param     compare: specifies the Capture Compare1 register new value
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_SetCompare2(TMR_T* TMRx, uint16_t compare)
{
    TMRx->CH2CC = compare;
}

/*!
 * @brief     Sets the Capture Compare3 Register value
 *
 * @param     TMRx: x can be can be 1 or 3 to select Timer
 *
 * @param     compare: specifies the Capture Compare1 register new value
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_SetCompare3(TMR_T* TMRx, uint16_t compare)
{
    TMRx->CH3CC = compare;
}

/*!
 * @brief     Sets the Capture Compare4 Register value
 *
 * @param     TMRx: x can be can be 1 or 3 to select Timer
 *
 * @param     compare: specifies the Capture Compare1 register new value
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_SetCompare4(TMR_T* TMRx, uint16_t compare)
{
    TMRx->CH4CC = compare;
}

/*!
 * @brief     Forces the output 1 waveform to active or inactive level
 *
 * @param     TMRx: x can be can be 1, 3, 14, 15, 16 and 17 to select Timer
 *
 * @param     action: forced Action to be set to the output waveform
 *                  This parameter can be any combination of the following values:
 *                     @arg TMR_FORCEDACTION_INACTIVE 
 *                     @arg TMR_FORCEDACTION_ACTIVE
 * @retval    None
 *
 * @note      None
 */
void TMR_ForcedOC1Config(TMR_T* TMRx, TMR_FORCED_ACTION_T action)
{
    TMRx->CH1CCM_OUTPUT_B.OC1MS = action;
}

/*!
 * @brief     Forces the output 2 waveform to active or inactive level
 *
 * @param     TMRx: x can be can be 1, 3 or 15 to select Timer
 *
 * @param     action: forced Action to be set to the output waveform
 * @param     action: forced Action to be set to the output waveform
 *                  This parameter can be any combination of the following values:
 *                     @arg TMR_FORCEDACTION_INACTIVE 
 *                     @arg TMR_FORCEDACTION_ACTIVE
 * @retval    None
 *
 * @note      None
 */
void TMR_ForcedOC2Config(TMR_T* TMRx, TMR_FORCED_ACTION_T action)
{
    TMRx->CH1CCM_OUTPUT_B.OC2MS = action;
}

/*!
 * @brief     Forces the output 3 waveform to active or inactive level
 *
 * @param     TMRx: x can be can be 1 or 3 to select Timer
 *
 * @param     action: forced Action to be set to the output waveform
 *                  This parameter can be any combination of the following values:
 *                     @arg TMR_FORCEDACTION_INACTIVE 
 *                     @arg TMR_FORCEDACTION_ACTIVE
 * @retval    None
 *
 * @note      None
 */
void TMR_ForcedOC3Config(TMR_T* TMRx, TMR_FORCED_ACTION_T action)
{
    TMRx->CH2CCM_OUTPUT_B.OC3MS = action;
}

/*!
 * @brief     Forces the output 4 waveform to active or inactive level
 *
 * @param     TMRx: x can be can be 1 or 3 to select Timer
 *
 * @param     action: forced Action to be set to the output waveform
 *                  This parameter can be any combination of the following values:
 *                     @arg TMR_FORCEDACTION_INACTIVE 
 *                     @arg TMR_FORCEDACTION_ACTIVE
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_ForcedOC4Config(TMR_T* TMRx, TMR_FORCED_ACTION_T action)
{
    TMRx->CH2CCM_OUTPUT_B.OC4MS = action;
}

/*!
 * @brief     Sets Capture Compare Preload Control bit
 *
 * @param     TMRx: x can be can be 1, 3 and 15 to select Timer
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_EnableCCPreload(TMR_T* TMRx)
{
    TMRx->CTRL2_B.CCBEN = ENABLE;
}

/*!
 * @brief     Resets Capture Compare Preload Control bit
 *
 * @param     TMRx: x can be can be 1, 3 and 15 to select Timer
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_DisableCCPreload(TMR_T* TMRx)
{
    TMRx->CTRL2_B.CCBEN = DISABLE;
}

/*!
 * @brief     Enables or disables the peripheral Preload register on CH1CC
 *
 * @param     TMRx: x can be can be 1, 3 ,14,15,16 and 17 to select Timer
 *
 * @param     OCPreload: new state of the TMRx peripheral Preload register
 *                  This parameter can be any combination of the following values:
 *                     @arg TMR_OC_PRELOAD_DISABLE 
 *                     @arg TMR_OC_PRELOAD_ENABLE
 * @retval    None
 *
 * @note      None
 */
void TMR_OC1PreloadConfig(TMR_T* TMRx, TMR_OC_PRELOAD_T OCPreload)
{
    TMRx->CH1CCM_OUTPUT_B.OC1BEN = OCPreload;
}

/*!
 * @brief     Enables or disables the peripheral Preload register on CH2CC
 *
 * @param     TMRx: x can be can be 1, 3 and 15 to select Timer
 *
 * @param     OCPreload: new state of the TMRx peripheral Preload register
 *                  This parameter can be any combination of the following values:
 *                     @arg TMR_OC_PRELOAD_DISABLE 
 *                     @arg TMR_OC_PRELOAD_ENABLE
 * @retval    None
 *
 * @note      None
 */
void TMR_OC2PreloadConfig(TMR_T* TMRx, TMR_OC_PRELOAD_T OCPreload)
{
    TMRx->CH1CCM_OUTPUT_B.OC2BEN = OCPreload;
}

/*!
 * @brief     Enables or disables the peripheral Preload register on CH3CC
 *
 * @param     TMRx: x can be can be 1 or 3  to select Timer
 *
 * @param     OCPreload: new state of the TMRx peripheral Preload register
 *                  This parameter can be any combination of the following values:
 *                     @arg TMR_OC_PRELOAD_DISABLE 
 *                     @arg TMR_OC_PRELOAD_ENABLE
 * @retval    None
 *
 * @note      None
 */
void TMR_OC3PreloadConfig(TMR_T* TMRx, TMR_OC_PRELOAD_T OCPreload)
{
    TMRx->CH2CCM_OUTPUT_B.OC3BEN = OCPreload;
}

/*!
 * @brief     Enables or disables the peripheral Preload register on CH4CC
 *
 * @param     TMRx: x can be can be 1 or 3  to select Timer
 *
 * @param     OCPreload: new state of the TMRx peripheral Preload register
 *                  This parameter can be any combination of the following values:
 *                     @arg TMR_OC_PRELOAD_DISABLE 
 *                     @arg TMR_OC_PRELOAD_ENABLE
 * @retval    None
 *
 * @note      None
 */
void TMR_OC4PreloadConfig(TMR_T* TMRx, TMR_OC_PRELOAD_T OCPreload)
{
    TMRx->CH2CCM_OUTPUT_B.OC4BEN = OCPreload;
}

/*!
 * @brief     Configures the Output Compare 1 Fast feature
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     OCFast: new state of the Output Compare Fast Enable Bit
 *                  This parameter can be any combination of the following values:
 *                     @arg TMR_OCFAST_DISABLE 
 *                     @arg TMR_OCFAST_ENABLE
 * @retval    None
 *
 * @note      None
 */

void TMR_OC1FastConfit(TMR_T* TMRx, TMR_OCFAST_T OCFast)
{
    TMRx->CH1CCM_OUTPUT_B.OC1FEN = OCFast;
}

/*!
 * @brief     Configures the Output Compare 2 Fast feature
 *
 * @param     TMRx: where x can be 1, 3, or 15 to select the TMR peripheral
 *
 * @param     OCFast: new state of the Output Compare Fast Enable Bit
 *                  This parameter can be any combination of the following values:
 *                     @arg TMR_OCFAST_DISABLE 
 *                     @arg TMR_OCFAST_ENABLE
 * @retval    None
 *
 * @note      None
 */
void TMR_OC2FastConfit(TMR_T* TMRx, TMR_OCFAST_T OCFast)
{
    TMRx->CH1CCM_OUTPUT_B.OC2FEN = OCFast;
}

/*!
 * @brief     Configures the Output Compare 3 Fast feature
 *
 * @param     TMRx: where x can be 1 or 3 to select the TMR peripheral
 *
 * @param     OCFast: new state of the Output Compare Fast Enable Bit
 *                  This parameter can be any combination of the following values:
 *                     @arg TMR_OCFAST_DISABLE 
 *                     @arg TMR_OCFAST_ENABLE
 * @retval    None
 *
 * @note      None
 */
void TMR_OC3FastConfit(TMR_T* TMRx, TMR_OCFAST_T OCFast)
{
    TMRx->CH2CCM_OUTPUT_B.OC3FEN = OCFast;
}

/*!
 * @brief     Configures the Output Compare 4 Fast feature
 *
 * @param     TMRx: where x can be 1 or 3 to select the TMR peripheral
 *
 * @param     OCFast: new state of the Output Compare Fast Enable Bit
 *                  This parameter can be any combination of the following values:
 *                     @arg TMR_OCFAST_DISABLE 
 *                     @arg TMR_OCFAST_ENABLE
 * @retval    None
 *
 * @note      None
 */
void TMR_OC4FastConfit(TMR_T* TMRx, TMR_OCFAST_T OCFast)
{
    TMRx->CH2CCM_OUTPUT_B.OC4FEN = OCFast;
}

/*!
 * @brief     Clears or safeguards the OCREF1 signal on an external event
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     OCCler: new state of the Output Compare Clear Enable Bit
 *                  This parameter can be any combination of the following values:
 *                     @arg TMR_OCCLER_DISABLE 
 *                     @arg TMR_OCCLER_ENABLE
 * @retval    None
 *
 * @note      None
 */
void TMR_ClearOC1Ref(TMR_T* TMRx, TMR_OCCLER_T OCCler)
{
    TMRx->CH1CCM_OUTPUT_B.OC1CEN = OCCler;
}

/*!
 * @brief     Clears or safeguards the OCREF2 signal on an external event
 *
 * @param     TMRx: where x can be 1, 3 or 15 to select the TMR peripheral
 *
 * @param     OCCler: new state of the Output Compare Clear Enable Bit
 *                  This parameter can be any combination of the following values:
 *                     @arg TMR_OCCLER_DISABLE 
 *                     @arg TMR_OCCLER_ENABLE
 * @retval    None
 *
 * @note      None
 */
void TMR_ClearOC2Ref(TMR_T* TMRx, TMR_OCCLER_T OCCler)
{
    TMRx->CH1CCM_OUTPUT_B.OC2CEN = OCCler;
}

/*!
 * @brief     Clears or safeguards the OCREF3 signal on an external event
 *
 * @param     TMRx: where x can be 1 or 3  to select the TMR peripheral
 *
 * @param     OCCler: new state of the Output Compare Clear Enable Bit
 *                  This parameter can be any combination of the following values:
 *                     @arg TMR_OCCLER_DISABLE 
 *                     @arg TMR_OCCLER_ENABLE
 * @retval    None
 *
 * @note      None
 */
void TMR_ClearOC3Ref(TMR_T* TMRx, TMR_OCCLER_T OCCler)
{
    TMRx->CH2CCM_OUTPUT_B.OC3CEN = OCCler;
}

/*!
 * @brief     Clears or safeguards the OCREF4 signal on an external event
 *
 * @param     TMRx: where x can be 1 or 3 to select the TMR peripheral
 *
 * @param     OCCler: new state of the Output Compare Clear Enable Bit
 *                  This parameter can be any combination of the following values:
 *                     @arg TMR_OCCLER_DISABLE 
 *                     @arg TMR_OCCLER_ENABLE
 * @retval    None
 *
 * @note      None
 */
void TMR_ClearOC4Ref(TMR_T* TMRx, TMR_OCCLER_T OCCler)
{
    TMRx->CH2CCM_OUTPUT_B.OC4CEN = OCCler;
}

/*!
 * @brief     Configures the  channel 1 polarity
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     OCPolarity: specifies the OC1 Polarity
 *                    This parameter can be any combination of the following values:
 *                     @arg TMR_OC_POLARITY_HIGH 
 *                     @arg TMR_OC_POLARITY_LOW
 * @retval    None
 *
 * @note      None
 */
void TMR_OC1PolarityConfig(TMR_T* TMRx, TMR_OC_POLARITY_T OCPolarity)
{
    TMRx->CHCTRL_B.CH1CCP = OCPolarity;
}

/*!
 * @brief     Configures the  channel 1N polarity
 *
 * @param     TMRx: where x can be 1, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     OCNPolarity: specifies the OC1 NPolarity
 *                    This parameter can be any combination of the following values:
 *                     @arg TMR_OC_NPOLARITY_HIGH 
 *                     @arg TMR_OC_NPOLARITY_LOW
 * @retval    None
 *
 * @note      None
 */
void TMR_OC1NPolarityConfig(TMR_T* TMRx, TMR_OC_NPOLARITY_T OCNPolarity)
{
    TMRx->CHCTRL_B.CH1OCNP = OCNPolarity;
}

/*!
 * @brief     Configures the  channel 2 polarity
 *
 * @param     TMRx: where x can be 1, 3 or 15 to select the TMR peripheral
 *
 * @param     OCPolarity: specifies the OC2 Polarity
 *                    This parameter can be any combination of the following values:
 *                     @arg TMR_OC_POLARITY_HIGH 
 *                     @arg TMR_OC_POLARITY_LOW
 * @retval    None
 *
 * @note      None
 */
void TMR_OC2PolarityConfig(TMR_T* TMRx, TMR_OC_POLARITY_T OCPolarity)
{
    TMRx->CHCTRL_B.CH2CCP = OCPolarity;
}

/*!
 * @brief     Configures the  channel 2N polarity
 *
 * @param     TMRx: where x can be 1  to select the TMR peripheral
 *
 * @param     OCNPolarity: specifies the OC2 NPolarity
 *                    This parameter can be any combination of the following values:
 *                     @arg TMR_OC_NPOLARITY_HIGH 
 *                     @arg TMR_OC_NPOLARITY_LOW
 * @retval    None
 *
 * @note      None
 */
void TMR_OC2NPolarityConfig(TMR_T* TMRx, TMR_OC_NPOLARITY_T OCNPolarity)
{
    TMRx->CHCTRL_B.CH2OCNP = OCNPolarity;
}

/*!
 * @brief     Configures the  channel 3 polarity
 *
 * @param     TMRx: where x can be 1, 3 to select the TMR peripheral
 *
 * @param     OCPolarity: specifies the OC3 Polarity
 *                    This parameter can be any combination of the following values:
 *                     @arg TMR_OC_POLARITY_HIGH 
 *                     @arg TMR_OC_POLARITY_LOW
 * @retval    None
 *
 * @note      None
 */
void TMR_OC3PolarityConfig(TMR_T* TMRx, TMR_OC_POLARITY_T OCPolarity)
{
    TMRx->CHCTRL_B.CH3CCP = OCPolarity;
}

/*!
 * @brief     Configures the  channel 3N polarity
 *
 * @param     TMRx: where x can be 1 to select the TMR peripheral
 *
 * @param     OCNPolarity: specifies the OC3 NPolarity
 *                    This parameter can be any combination of the following values:
 *                     @arg TMR_OC_NPOLARITY_HIGH 
 *                     @arg TMR_OC_NPOLARITY_LOW
 * @retval    None
 *
 * @note      None
 */
void TMR_OC3NPolarityConfig(TMR_T* TMRx, TMR_OC_NPOLARITY_T OCNPolarity)
{
    TMRx->CHCTRL_B.CH3OCNP = OCNPolarity;
}

/*!
 * @brief     Configures the  channel 4 polarity
 *
 * @param     TMRx: where x can be 1, 3 to select the TMR peripheral
 *
 * @param     OCPolarity: specifies the OC4 Polarity
 *                    This parameter can be any combination of the following values:
 *                     @arg TMR_OC_POLARITY_HIGH 
 *                     @arg TMR_OC_POLARITY_LOW
 * @retval    None
 *
 * @note      None
 */
void TMR_OC4PolarityConfig(TMR_T* TMRx, TMR_OC_POLARITY_T OCPolarity)
{
    TMRx->CHCTRL_B.CH4CCP = OCPolarity;
}

/*!
 * @brief     Selects the OCReference Clear source
 *
 * @param     TMRx: x can be  1, 3 to select Timer.
 *
 * @param     OCReferenceClear: specifies the OCReference Clear source
 *                    This parameter can be any combination of the following values:
 *                     @arg TMR_OCCS_ETRF 
 *                     @arg TMR_OCCS_OCREFCLR
 * @retval    None
 *
 * @note      None
 */
void TMR_SelectOCREFClear(TMR_T* TMRx, TMR_OCCSEL_T OCReferenceClear)
{
    TMRx->SMC_B.OCCSEL = OCReferenceClear;
}

/*!
 * @brief     Enables the Capture Compare Channel x
 *
 * @param    TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     channel: TMR channel
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_EnableCCxChannel(TMR_T* TMRx, TMR_CHANNEL_T channel)
{
    TMRx->CHCTRL |= BIT_SET << channel;
}

/*!
 * @brief     Disables the Capture Compare Channel x
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     channel: TMR channel
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_DisableCCxChannel(TMR_T* TMRx, TMR_CHANNEL_T channel)
{
    TMRx->CHCTRL &= BIT_RESET << channel;
}

/*!
 * @brief     Enables the Capture Compare Channel xN.
 *
 * @param     TMRx: where x can be 1, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     channel: TMR channel
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_EnableCCxNChannel(TMR_T* TMRx, TMR_CHANNEL_T channel)
{
    TMRx->CHCTRL |= 0x04 << channel;
}

/*!
 * @brief     Disables the Capture Compare Channel xN
 *
 * @param     TMRx: where x can be 1, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     channel: TMR channel
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_DisableCCxNChannel(TMR_T* TMRx, TMR_CHANNEL_T channel)
{
    TMRx->CHCTRL &= BIT_RESET << channel;
}

/*!
 * @brief     Enable Selects the TMR peripheral Commutation event
 *
 * @param     TMRx: where x can be 1, 15, 16 or 17 to select the TMR peripheral
 *
 * @retval    None
 *
 * @note      None
 */

void TMR_EnableSelectCOM(TMR_T* TMRx)
{
    TMRx->CTRL2_B.CCUS = ENABLE;
}
/*!
 * @brief     Disable Selects the TMR peripheral Commutation event
 *
 * @param     TMRx: where x can be 1, 15, 16 or 17 to select the TMR peripheral
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_DisableSelectCOM(TMR_T* TMRx)
{
    TMRx->CTRL2_B.CCUS = DISABLE;
}

/*!
 * @brief     Configure the TI1 as Input.
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     ICpolarity: The Input Polarity.
 *
 * @param     ICselection: specifies the input to be used.
 *
 * @param     ICfilter: Specifies the Input Capture Filter
 *
 * @retval    None
 *
 * @note      None
 */
static void TI1Config(TMR_T* TMRx, uint16_t ICpolarity, uint16_t ICselection, uint16_t ICfilter)
{
    uint16_t tmpchctrl = 0;

    /** Disable the Channel 1: Reset the CH1CCEN Bit */
    TMRx->CHCTRL_B.CH1CCEN = BIT_RESET;

    /** Select the Input and set the filter */
    TMRx->CH1CCM_INPUT_B.MOD1SEL = BIT_RESET;
    TMRx->CH1CCM_INPUT_B.IC1FC = BIT_RESET;
    TMRx->CH1CCM_INPUT_B.MOD1SEL = ICselection;
    TMRx->CH1CCM_INPUT_B.IC1FC = ICfilter;

    /** Select the Polarity */
    tmpchctrl = TMRx->CHCTRL;
    tmpchctrl = ICpolarity;
    TMRx->CHCTRL = tmpchctrl;

    /** Set the CH1CCEN Bit */
    TMRx->CHCTRL_B.CH1CCP = BIT_RESET;
    TMRx->CHCTRL_B.CH1OCNP = BIT_RESET;
    TMRx->CHCTRL_B.CH1CCEN = BIT_SET;
}

/*!
 * @brief     Configure the TI2 as Input
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR periphera
 *
 * @param     ICpolarity: The Input Polarity.
 *
 * @param     ICselection: specifies the input to be used.
 *
 * @param     ICfilter: Specifies the Input Capture Filter
 *
 * @retval    None
 *
 * @note      None
 */
static void TI2Config(TMR_T* TMRx, uint16_t ICpolarity, uint16_t ICselection, uint16_t ICfilter)
{
    uint16_t tmpchctrl = 0;

    /** Disable the Channel 2: Reset the CH2CCEN Bit */
    TMRx->CHCTRL_B.CH2CCEN = BIT_RESET;

    /** Select the Input and set the filter */
    TMRx->CH1CCM_INPUT_B.MOD2SEL = BIT_RESET;
    TMRx->CH1CCM_INPUT_B.IC2FC = BIT_RESET;
    TMRx->CH1CCM_INPUT_B.MOD2SEL = ICselection;
    TMRx->CH1CCM_INPUT_B.IC2FC = ICfilter;

    /** Select the Polarity */
    tmpchctrl = TMRx->CHCTRL;
    tmpchctrl = (uint16_t)(ICpolarity << 4);
    TMRx->CHCTRL = tmpchctrl;

    /* Set the CH2CCEN Bit */
    TMRx->CHCTRL_B.CH2CCP = BIT_RESET;
    TMRx->CHCTRL_B.CH2OCNP = BIT_RESET;
    TMRx->CHCTRL_B.CH2CCEN = BIT_SET;
}

/*!
 * @brief     Configure the TI3 as Input.
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     ICpolarity: The Input Polarity.
 *
 * @param     ICselection: specifies the input to be used.
 *
 * @param     ICfilter: Specifies the Input Capture Filter
 *
 * @retval    None
 *
 * @note      None
 */
static void TI3Config(TMR_T* TMRx, uint16_t ICpolarity, uint16_t ICselection, uint16_t ICfilter)
{
    uint16_t tmpchctrl = 0;

    /** Disable the Channel 3: Reset the CH3CCEN Bit */
    TMRx->CHCTRL_B.CH3CCEN = BIT_RESET;

    /** Select the Input and set the filter */
    TMRx->CH2CCM_INPUT_B.MOD3SEL = BIT_RESET;
    TMRx->CH2CCM_INPUT_B.IC3FC = BIT_RESET;
    TMRx->CH2CCM_INPUT_B.MOD3SEL = ICselection;
    TMRx->CH2CCM_INPUT_B.IC3FC = ICfilter;

    /** Select the Polarity */
    tmpchctrl = TMRx->CHCTRL;
    tmpchctrl = (uint16_t)(ICpolarity << 8);
    TMRx->CHCTRL = tmpchctrl;

    /* Set the CH3CCEN Bit */
    TMRx->CHCTRL_B.CH3CCP = BIT_RESET;
    TMRx->CHCTRL_B.CH3OCNP = BIT_RESET;
    TMRx->CHCTRL_B.CH3CCEN = BIT_SET;
}

/*!
 * @brief     Configure the TI4 as Input
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     ICpolarity: The Input Polarity.
 *
 * @param     ICselection: specifies the input to be used.
 *
 * @param     ICfilter: Specifies the Input Capture Filter
 *
 * @retval    None
 *
 * @note      None
 */
static void TI4Config(TMR_T* TMRx, uint16_t ICpolarity, uint16_t ICselection, uint16_t ICfilter)
{
    uint16_t tmpchctrl = 0;

    /** Disable the Channel 4: Reset the CH4CCEN Bit */
    TMRx->CHCTRL_B.CH4CCEN = BIT_RESET;

    /** Select the Input and set the filter */
    TMRx->CH2CCM_INPUT_B.MOD4SEL = BIT_RESET;
    TMRx->CH2CCM_INPUT_B.IC4FC = BIT_RESET;
    TMRx->CH2CCM_INPUT_B.MOD4SEL = ICselection;
    TMRx->CH2CCM_INPUT_B.IC4FC = ICfilter;

    /** Select the Polarity */
    tmpchctrl = TMRx->CHCTRL;
    tmpchctrl = (uint16_t)(ICpolarity << 12);
    TMRx->CHCTRL = tmpchctrl;

    /* Set the CH4CCEN Bit */
    TMRx->CHCTRL_B.CH4CCP = BIT_RESET;
    TMRx->CHCTRL_B.CH4CCEN = BIT_SET;
}

/*!
 * @brief     Configure Peripheral equipment
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     ICconfigstruct: pointer to a TMR_ICConfig_T structure
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_ConfigIC(TMR_T* TMRx, TMR_ICConfig_T* ICconfigstruct)
{
    if (ICconfigstruct->channel == TMR_CHANNEL_1)
    {
        /** TI1 Configuration */
        TI1Config(TMRx, ICconfigstruct->ICpolarity, ICconfigstruct->ICselection, ICconfigstruct->ICfilter);
        TMR_SetIC1Prescal(TMRx, ICconfigstruct->ICprescaler);
    }
    else if (ICconfigstruct->channel == TMR_CHANNEL_2)
    {
        /** TI2 Configuration */
        TI2Config(TMRx, ICconfigstruct->ICpolarity, ICconfigstruct->ICselection, ICconfigstruct->ICfilter);
        TMR_SetIC2Prescal(TMRx, ICconfigstruct->ICprescaler);
    }
    else if (ICconfigstruct->channel == TMR_CHANNEL_3)
    {
        /** TI3 Configuration */
        TI3Config(TMRx, ICconfigstruct->ICpolarity, ICconfigstruct->ICselection, ICconfigstruct->ICfilter);
        TMR_SetIC3Prescal(TMRx, ICconfigstruct->ICprescaler);
    }
    else if (ICconfigstruct->channel == TMR_CHANNEL_4)
    {
        /** TI4 Configuration */
        TI4Config(TMRx, ICconfigstruct->ICpolarity, ICconfigstruct->ICselection, ICconfigstruct->ICfilter);
        TMR_SetIC4Prescal(TMRx, ICconfigstruct->ICprescaler);
    }
}

/*!
 * @brief     Initialize the IC timer with its default value.
 *
 * @param     ICconfigstruct: pointer to a TMR_ICConfig_T structure
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_ConfigICStructInit(TMR_ICConfig_T* ICconfigstruct)
{
    /** Set the default configuration */
    ICconfigstruct->channel = TMR_CHANNEL_1;
    ICconfigstruct->ICpolarity = TMR_IC_POLARITY_RISING;
    ICconfigstruct->ICselection = TMR_IC_SELECTION_DIRECT_TI;
    ICconfigstruct->ICprescaler = TMR_ICPSC_DIV1;
    ICconfigstruct->ICfilter = 0x00;
}

/*!
 * @brief     Config of PWM output
 *
 * @param     TMRx: where x can be 1, 3, or 15 to select the TMR peripheral
 *
 * @param     ICconfigstruct: pointer to a TMR_ICConfig_T structure
 *
 * @retval    None
 *
 * @note      x can be  1, 2, 3, 4, 5, 8, 9, 12 or 15 to select .
 */
void TMR_ConfigPWM(TMR_T* TMRx, TMR_ICConfig_T* ICconfigstruct)
{
    uint16_t icpolarity = TMR_IC_POLARITY_RISING;
    uint16_t icselection = TMR_IC_SELECTION_DIRECT_TI;

    /** Select the Opposite Input Polarity */
    if (ICconfigstruct->ICpolarity == TMR_IC_POLARITY_RISING)
    {
        icpolarity = TMR_IC_POLARITY_FALLING;
    }
    else
    {
        icpolarity = TMR_IC_POLARITY_RISING;
    }

    /** Select the Opposite Input */
    if (ICconfigstruct->ICselection == TMR_IC_SELECTION_DIRECT_TI)
    {
        icselection = TMR_IC_SELECTION_INDIRECT_TI;
    }
    else
    {
        icselection = TMR_IC_SELECTION_DIRECT_TI;
    }

    if (ICconfigstruct->channel == TMR_CHANNEL_1)
    {
        /** TI1 Configuration */
        TI1Config(TMRx, ICconfigstruct->ICpolarity, ICconfigstruct->ICselection, ICconfigstruct->ICfilter);
        /** Set the Input Capture Prescaler value */
        TMR_SetIC1Prescal(TMRx, ICconfigstruct->ICprescaler);
        /** TI2 Configuration */
        TI2Config(TMRx, icpolarity, icselection, ICconfigstruct->ICfilter);
        /** Set the Input Capture Prescaler value */
        TMR_SetIC2Prescal(TMRx, ICconfigstruct->ICprescaler);
    }
    else
    {
        /** TI2 Configuration */
        TI2Config(TMRx, ICconfigstruct->ICpolarity, ICconfigstruct->ICselection, ICconfigstruct->ICfilter);
        /** Set the Input Capture Prescaler value */
        TMR_SetIC2Prescal(TMRx, ICconfigstruct->ICprescaler);
        /** TI1 Configuration */
        TI1Config(TMRx, icpolarity, icselection, ICconfigstruct->ICfilter);
        /** Set the Input Capture Prescaler value */
        TMR_SetIC1Prescal(TMRx, ICconfigstruct->ICprescaler);
    }
}

/*!
 * @brief     Read Input Capture 1 value
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @retval    Capture Compare 1 Register value
 *
 * @note      None
 */
uint16_t TMR_ReadCaputer1(TMR_T* TMRx)
{
    return TMRx->CH1CC;
}

/*!
 * @brief     Read Input Capture 2 value
 *
 * @param     TMRx: where x can be 1, 3 or 15 to select the TMR peripheral
 *
 * @retval    Capture Compare 2 Register value
 *
 * @note      None
 */
uint16_t TMR_ReadCaputer2(TMR_T* TMRx)
{
    return TMRx->CH2CC;
}

/*!
 * @brief     Read Input Capture 3 value
 *
 * @param     TMRx: where x can be 1, 3  to select the TMR peripheral
 *
 * @retval    Capture Compare 3 Register value
 *
 * @note      None
 */
uint16_t TMR_ReadCaputer3(TMR_T* TMRx)
{
    return TMRx->CH3CC;
}

/*!
 * @brief     Read Input Capture 4 value
 *
 * @param     TMRx: where x can be 1, 3  to select the TMR peripheral
 *
 * @retval    Capture Compare 4 Register value
 *
 * @note      None
 */
uint16_t TMR_ReadCaputer4(TMR_T* TMRx)
{
    return TMRx->CH4CC;
}

/*!
 * @brief     Sets the TMRx Input Capture 1 prescaler
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     prescaler: specifies the Input Capture 1 prescaler new value
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_SetIC1Prescal(TMR_T* TMRx, TMR_IC_PRESCALER_T prescaler)
{
    TMRx->CH1CCM_INPUT_B.IC1D = BIT_RESET;
    TMRx->CH1CCM_INPUT_B.IC1D = prescaler;
}
/*!
 * @brief     Sets the TMRx Input Capture 2 prescaler
 *
 * @param     TMRx: where x can be 1, 3, or 15 to select the TMR peripheral
 *
 * @param     prescaler: specifies the Input Capture 2 prescaler new value
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_SetIC2Prescal(TMR_T* TMRx, TMR_IC_PRESCALER_T prescaler)
{
    TMRx->CH1CCM_INPUT_B.IC2D = BIT_RESET;
    TMRx->CH1CCM_INPUT_B.IC2D = prescaler;
}

/*!
 * @brief     Sets the TMRx Input Capture 3 prescaler
 *
 * @param     TMRx: where x can be 1 or 3 to select the TMR peripheral
 *
 * @param     prescaler: specifies the Input Capture 3 prescaler new value
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_SetIC3Prescal(TMR_T* TMRx, TMR_IC_PRESCALER_T prescaler)
{
    TMRx->CH2CCM_INPUT_B.IC3D = BIT_RESET;
    TMRx->CH2CCM_INPUT_B.IC3D = prescaler;
}

/*!
 * @brief     Sets the TMRx Input Capture 4 prescaler
 *
 * @param     TMRx: where x can be 1 or 3 to select the TMR peripheral
 *
 * @param     prescaler: specifies the Input Capture 4 prescaler new value
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_SetIC4Prescal(TMR_T* TMRx, TMR_IC_PRESCALER_T prescaler)
{
    TMRx->CH2CCM_INPUT_B.IC4D = BIT_RESET;
    TMRx->CH2CCM_INPUT_B.IC4D = prescaler;
}

/*!
 * @brief     Enable intterupts
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     sources: specifies the TMR interrupts sources
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_EnableINT(TMR_T* TMRx, TMR_IT_SOURCES_T sources)
{
    TMRx->INTCTRL |= sources;
}

/*!
 * @brief     Disable intterupts
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     sources: specifies the TMR interrupts sources
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_DisableINT(TMR_T* TMRx, TMR_IT_SOURCES_T sources)
{
    TMRx->INTCTRL &= ~sources;
}

/*!
 * @brief     Configures the TMRx event to be generate by software
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     sources: specifies the event source
 *
 * @retval    None
 *
 * @note      TMR_EVENT_CCU and TMR_EVENT_BRK are used only with TMR1
 */
void TMR_GenerateEvent(TMR_T* TMRx, TMR_EVENT_SOURCES_T sources)
{
    /** Set the event sources */
    TMRx->SCEG = sources;
}

/*!
 * @brief     Check whether the flag is set or reset
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     flag: specifies the flag to check
 *
 * @retval    The new state of flag
 *
 * @note      TMR15 can have only TMR_FLAG_UDIF, TMR_FLAG_CH1CCIF, TMR_FLAG_CH2CCIF or TMR_FLAG_TRGIF.
 *            TMR14, TMR16 and TMR17 can have TMR_FLAG_UDIF or TMR_FLAG_CH1CCIF
 *            TMR_FLAG_BRKIF is used only with TMR1 and TMR15.
 *            TMR_FLAG_CCUIF is used only with TMR1 TMR15, TMR16 and TMR17
 */
uint16_t TMR_ReadStatusFlagState(TMR_T* TMRx, TMR_FLAG_T flag)
{
    if ((TMRx->STS & flag) != RESET)
    {
        return SET;
    }
    else
    {
        return RESET;
    }
}

/*!
 * @brief     Clears the TMR's pending flags
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     flag: Flags that need to be clear
 *
 * @retval    None
 *
 * @note      TMR15 can have only TMR_FLAG_UDIF, TMR_FLAG_CH1CCIF, TMR_FLAG_CH2CCIF or TMR_FLAG_TRGIF.
 *            TMR14, TMR16 and TMR17 can have TMR_FLAG_UDIF or TMR_FLAG_CH1CCIF
 *            TMR_FLAG_BRKIF is used only with TMR1 and TMR15
 *            TMR_FLAG_CCUIF is used only with TMR1 TMR15, TMR16 and TMR17
 */
void TMR_ClearStatusFlag(TMR_T* TMRx, TMR_FLAG_T flag)
{
    TMRx->STS = ~flag;
}

/*!
 * @brief     Check whether the ITflag is set or reset
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     sources: interrupt source to check.
 *
 * @retval    The new state of the ITflag
 *
 * @note      TMR15 can have only TMR_TI_UPDATE, TMR_TI_CH1, TMR_TI_CH2 or TMR_TI_TRG
 *            TMR14, TMR16 and TMR17 can have TMR_TI_UPDATE or TMR_TI_CH1
 *            TMR_TI_BRK is used only with TMR1 and TMR15.
 *            TMR_TI_CCU is used only with TMR1, TMR15, TMR16 and TMR17
 */
uint16_t TMR_ReadIntFlag(TMR_T* TMRx,  TMR_IT_SOURCES_T sources)
{
    if (((TMRx->STS & sources) != RESET ) && ((TMRx->INTCTRL & sources) != RESET))
    {
        return SET;
    }
    else
    {
        return RESET;
    }
}

/*!
 * @brief     Clears the TMR's interrupt pending bits
 *
 * @param     TMRx: where x can be 1, 3, 14, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     IT_Sources:interrupt source to Clear
 *
 * @retval    None
 *
 * @note      TMR15 can have only TMR_TI_UPDATE, TMR_TI_CH1, TMR_TI_CH2 or TMR_TI_TRG
 *            TMR14, TMR16 and TMR17 can have TMR_TI_UPDATE or TMR_TI_CH1
 *            TMR_TI_BRK is used only with TMR1 and TMR15.
 *            TMR_TI_CCU is used only with TMR1, TMR15, TMR16 and TMR17
 */
void TMR_ClearIntFlag(TMR_T* TMRx,  TMR_IT_SOURCES_T sources)
{
    TMRx->STS = ~sources;
}

/*!
 * @brief     Configures the TMRx's DMA interface.
 *
 * @param     TMRx: where x can be 1, 3, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     address: DMA Base address
 *
 * @param     lenght: DMA Burst length
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_ConfigDMA(TMR_T* TMRx, TMR_DMA_BASE_ADDERSS_T address, TMR_DMA_BURST_LENGHT_T lenght)
{
    /** Set the DMA Base and the DMA Burst Length */
    TMRx->DCTRL = address | lenght;

}

/*!
 * @brief     Enable TMRx Requests
 *
 * @param     TMRx: where x can be 1, 3, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     souces: specifies the DMA Request sources
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_EnableDMASoure(TMR_T* TMRx, TMR_DMA_SOUCES_T souces)
{
    TMRx->INTCTRL |= souces;
}

/*!
 * @brief     Disable TMRx Requests
 *
 * @param     TMRx: where x can be 1, 3, 6, 15, 16 or 17 to select the TMR peripheral
 *
 * @param     DMAsouces: specifies the DMA Request sources
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_DisableDMASoure(TMR_T* TMRx, TMR_DMA_SOUCES_T souces)
{
    TMRx->INTCTRL &= ~souces;
}

/*!
 * @brief     Enable Capture Compare DMA source
 *
 * @param     TMRx: where x can be 1, 3, 15, 16 or 17 to select the TMR peripheral
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_EnableCCDMA(TMR_T* TMRx)
{
    TMRx->CTRL2_B.CCDS = ENABLE;
}

/*!
 * @brief     Disable Capture Compare DMA source
 *
 * @param     TMRx: where x can be 1, 3, 15, 16 or 17 to select the TMR peripheral
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_DisableCCDMA(TMR_T* TMRx)
{
    TMRx->CTRL2_B.CCDS = DISABLE;
}

/*!
 * @brief     Configures the TMRx internal Clock
 *
 * @param     TMRx: where x can be 1, 3 or 15 select the TMR peripheral
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_ConfigInternalClock(TMR_T* TMRx)
{
    TMRx->SMC_B.SMFC = DISABLE;
}

/*!
 * @brief     Configures the TMRx Internal Trigger as External Clock
 *
 * @param     TMRx: where x can be 1, 3 or 15 select the TMR peripheral
 *
 * @param     inputTriggerSouce:Trigger source.
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_ConfigITRxExternalClock(TMR_T* TMRx, TMR_INPUT_TRIGGER_SOURCE_T input)
{
    TMR_SelectInputTrigger(TMRx, input);
    TMRx->SMC_B.SMFC = 0x07;
}

/*!
 * @brief     Configures the TMRx  Trigger as External Clock
 *
 * @param     TMRx: where x can be 1, 3 or 15 select the TMR peripheral
 *
 * @param     TIx_external_clksource: Trigger source.
 *
 * @param     IC_polarity:specifies the TIx Polarity.
 *
 * @param     ICfilter:specifies the filter value.This parameter must be a value between 0x0 and 0xF.
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_TIxExternalClockConfig(TMR_T* TMRx, TMR_INPUT_TRIGGER_SOURCE_T TIx_external_clksource,
                                TMR_IC_POLARITY_T IC_polarity, uint16_t ICfilter)
{
    /** Configure the Timer Input Clock Source */
    if (TIx_external_clksource == 0x06)
    {
        TI2Config(TMRx, IC_polarity, TMR_IC_SELECTION_DIRECT_TI, ICfilter);
    }
    else
    {
        TI1Config(TMRx, IC_polarity, TMR_IC_SELECTION_DIRECT_TI, ICfilter);
    }

    /** Select the Trigger source */
    TMR_SelectInputTrigger(TMRx, TIx_external_clksource);
    /** Select the External clock mode1 */
    TMRx->SMC_B.SMFC = 0x07;
}

/*!
 * @brief     Configures the External clock Mode1
 *
 * @param     TMRx: where x can be 1 or 3 select the TMR peripheral
 *
 * @param     prescaler: The external Trigger Prescaler.
 *
 * @param     polarity: The external Trigger Polarity.
 *
 * @param     filter: External Trigger Filter.
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_ETRClockMode1Config(TMR_T* TMRx, TMR_EXTTRG_PRESCALER_T prescaler,
                             TMR_EXTTRG_POLARITY_T polarity, uint16_t filter)
{
    /** Configure the ETR Clock source */
    TMR_ConfigETR(TMRx, prescaler, polarity, filter);
    /** Reset the SMFC Bits */
    TMRx->SMC_B.SMFC = BIT_RESET;
    /** Select the External clock mode1 */
    TMRx->SMC_B.SMFC = 0x07;
    /** Select the Trigger selection : ETF */
    TMRx->SMC_B.ITC = 0x07;
}

/*!
 * @brief     Configures the External clock Mode2
 *
 * @param     TMRx: where x can be 1 or 3 select the TMR peripheral
 *
 * @param     prescaler: The external Trigger Prescaler
 *
 * @param     polarity: The external Trigger Polarity
 *
 * @param     filter: External Trigger Filter
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_ConfigETRClockMode2(TMR_T* TMRx, TMR_EXTTRG_PRESCALER_T prescaler,
                             TMR_EXTTRG_POLARITY_T polarity, uint16_t filter)
{
    /** Configure the ETR Clock source */
    TMR_ConfigETR(TMRx, prescaler, polarity, filter);
    /** Enable the External clock mode2 */
    TMRx->SMC_B.ECM2EN = ENABLE;
}

/*!
 * @brief     Selects the Input Trigger source
 *
 * @param     TMRx: where x can be 1, 3 or 15 select the TMR peripheral
 *
 * @param     inputTriggerSource: Input Trigger source
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_SelectInputTrigger(TMR_T* TMRx, TMR_INPUT_TRIGGER_SOURCE_T source)
{
    TMRx->SMC_B.ITC = BIT_RESET;
    TMRx->SMC_B.ITC = source;
}

/*!
 * @brief     Selects the Trigger Output Mode.
 *
 * @param     TMRx: where x can be 1, 3, 6, or 15 select the TMR peripheral
 *
 * @param     source specifies the Trigger Output source.
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_SelectOutputTrigger(TMR_T* TMRx, TMR_TRGOSOURCE_T source)
{
    TMRx->CTRL2_B.MMFC = source;
}

/*!
 * @brief     Selects the Slave Mode.
 *
 * @param     TMRx: where x can be 1, 3 or 15 select the TMR peripheral
 *
 * @param     slaveMode:Slave Mode.
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_SelectSlaveMode(TMR_T* TMRx, TMR_SLAVEMODE_T mode)
{
    TMRx->SMC_B.SMFC = mode;
}

/*!
 * @brief     Enable the Master Slave Mode
 *
 * @param     TMRx: where x can be 1, 3 or 15 select the TMR peripheral
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_EnableMasterSlaveMode(TMR_T* TMRx)
{
    TMRx->SMC_B.MSMEN = ENABLE ;
}

/*!
 * @brief     Disable the Master Slave Mode
 *
 * @param     TMRx: where x can be 1, 3 or 15 select the TMR peripheral
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_DisableMasterSlaveMode(TMR_T* TMRx)
{
    TMRx->SMC_B.MSMEN = DISABLE ;
}

/*!
 * @brief     Configures the TMRx External Trigger (ETR)
 *
 * @param     TMRx: where x can be 1 or 3 select the TMR peripheral
 *
 * @param     prescaler: The external Trigger Prescaler
 *
 * @param     polarity: The external Trigger Polarity
 *
 * @param     filter: External Trigger Filter
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_ConfigETR(TMR_T* TMRx, TMR_EXTTRG_PRESCALER_T prescaler,
                   TMR_EXTTRG_POLARITY_T polarity, uint16_t filter)
{
    /** Reset the ETR Bits */
    TMRx->SMC &= 0x00FF;
    /* Set the Prescaler, the Filter value and the Polarity */
    TMRx->SMC_B.ETDC = prescaler;
    TMRx->SMC_B.ETPC = polarity;
    TMRx->SMC_B.ETFC = filter;
}

/*!
 * @brief     Configures the Encoder Interface
 *
 * @param     TMRx: where x can be 1 or 3 select the TMR peripheral
 *
 * @param     encodeMode: specifies the Encoder Mode
 *
 * @param     IC1Polarity: specifies the IC1 Polarity
 *
 * @param     IC2Polarity: specifies the IC2 Polarity
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_ConfigEncodeInterface(TMR_T* TMRx, TMR_ENCODER_MODE_T encodeMode, TMR_IC_POLARITY_T IC1Polarity,
                               TMR_IC_POLARITY_T IC2Polarity)
{
    /** Set the encoder Mode */
    TMRx->SMC_B.SMFC = BIT_RESET;
    TMRx->SMC_B.SMFC = encodeMode;

    /** Select the Capture Compare 1 and the Capture Compare 2 as input */
    TMRx->CH1CCM_INPUT_B.MOD1SEL = BIT_RESET ;
    TMRx->CH1CCM_INPUT_B.MOD2SEL = BIT_RESET ;
    TMRx->CH1CCM_INPUT_B.MOD1SEL = BIT_SET ;
    TMRx->CH1CCM_INPUT_B.MOD2SEL = BIT_SET ;

    /** Set the TI1 and the TI2 Polarities */
    TMRx->CHCTRL_B.CH1CCP = BIT_RESET;
    TMRx->CHCTRL_B.CH2CCP = BIT_RESET;
    TMRx->CHCTRL_B.CH1CCP = IC1Polarity;
    TMRx->CHCTRL_B.CH2CCP = IC2Polarity;
}

/*!
 * @brief     Enables Hall sensor interface.
 *
 * @param     TMRx: where x can be 1 or 3 select the TMR peripheral
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_EnableHallSensor(TMR_T* TMRx)
{
    TMRx->CTRL2_B.TI1SEL = ENABLE;
}

/*!
 * @brief     Disable Hall sensor interface.
 *
 * @param     TMRx: where x can be 14 to select the TMR peripheral
 *
 * @retval    None
 *
 * @note      None
 */
void TMR_DisableHallSensor(TMR_T* TMRx)
{
    TMRx->CTRL2_B.TI1SEL = DISABLE;
}


/**@} end of group TMR_Fuctions*/
/**@} end of group TMR_Driver */
/**@} end of group Peripherals_Library*/
